/*

 #    #    ##       #    #    #           ####
 ##  ##   #  #      #    ##   #          #    #
 # ## #  #    #     #    # #  #          #
 #    #  ######     #    #  # #   ###    #
 #    #  #    #     #    #   ##   ###    #    #
 #    #  #    #     #    #    #   ###     ####

	The main entry point.
*/

/*
 * $Id: main.c,v 2.9 92/07/14 16:48:27 ram Exp $
 *
 *  Copyright (c) 1992, Raphael Manfredi
 *
 *  You may redistribute only under the terms of the GNU General Public
 *  Licence as specified in the README file that comes with dist.
 *
 * $Log:	main.c,v $
 * Revision 2.9  92/07/14  16:48:27  ram
 * 3.0 beta baseline.
 * 
 */

#include "config.h"
#include "portable.h"

#ifdef I_STRING
#include <string.h>
#else
#include <strings.h>
#endif

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include "logfile.h"
#include "io.h"
#include "hash.h"
#include "msg.h"
#include "parser.h"
#include "sysexits.h"

#define MAX_STRING	2048	/* Maximum string length */

private Signal_t handler();	/* Signal handler */
private void set_signal();	/* Set up the signal handler */

extern void env_home();		/* Only for tests */

public void main(argc, argv, envp)
int argc;
char **argv;
char **envp;
{
	/* This is the main entry point for the mail filter */

	char logfile[MAX_STRING];			/* Location of logfile */
	char *value;						/* Symbol value */
	int level = 0;						/* Logging level wanted */
	int euid, uid;						/* Current effective and real uid */

	/* Compute program name, removing any leading path to keep only the name
	 * of the executable file.
	 */
	progname = rindex(argv[0], '/');	/* Only last name if '/' found */
	if (progname++ == (char *) 0)		/* There were no '/' */
		progname = argv[0];				/* This must be the filename then */
	progpid = getpid();					/* Program's PID */

	/* The '-t' option means we are in test mode: set the home directory by
	 * using the environment HOME variable, so that we may provide our own
	 * configuration file elsewhere.
	 */
	if (argc > 1 && 0 == strcmp(argv[1], "-t"))
		env_home();						/* Get HOME form environment */

	set_signal();						/* Set up signal handler */
	read_conf(".mailagent");			/* Read configuration file */

	value = ht_value(&symtab, "logdir");	/* Fetch logging directory */
	if (value == (char *) 0)
		fatal("logging directory not defined");
	strcpy(logfile, value);
	strcat(logfile, "/");
	value = ht_value(&symtab, "log");
	if (value == (char *) 0)
		fatal("logfile not defined");
	strcat(logfile, value);

	value = ht_value(&symtab, "level");		/* Fetch logging level */
	if (value == (char *) 0)
		fatal("no logging level defined");
	sscanf(value, "%d", &level);

	set_loglvl(level);						/* Logging level wanted */
	if (-1 == open_log(logfile))
		fprintf(stderr, "%s: cannot open logfile %s\n", progname, logfile);

	add_log(19, "starting processing");

	/* We'll be invoking a perl script with the -S switch, and perl will not
	 * allow us to do that if it detects "setuidness". Some sendmail programs
	 * are broken and do not reset the uid/gid correctly when they process
	 * their queue. This is why it is important to set the setuid and setgid
	 * bits on the filter program.
	 */

	/* Make sure our gid matches the effective gid */
	euid = getegid();
	uid = getgid();
	if (euid != uid && -1 == setgid(euid)) {
		add_log(1, "SYSERR setgid: %m (%e)");
		add_log(4, "WARNING cannot set GID to %d, continuing as %d", euid, uid);
	} else if (euid != uid)
		add_log(6, "NOTICE reset GID from %d to %d", uid, euid);

	/* Make sure our uid matches the effective uid */
	euid = geteuid();
	uid = getuid();
	if (euid != uid && -1 == setuid(euid)) {
		add_log(1, "SYSERR setuid: %m (%e)");
		add_log(4, "WARNING cannot set UID to %d, continuing as %d", euid, uid);
	} else if (euid != uid)
		add_log(6, "NOTICE reset UID from %d to %d", uid, euid);

	value = ht_value(&symtab, "queue");		/* Fetch queue location */
	if (value == (char *) 0)
		fatal("queue directory not defined");

	set_env_vars(envp);						/* Set environment variables */
	process();								/* Process mail */

	exit(EX_OK);		/* We did it */
}

private void set_signal()
{
	/* Set up the signal handler */

#ifdef SIGHUP
	signal(SIGHUP, handler);
#endif
#ifdef SIGINT
	signal(SIGINT, handler);
#endif
#ifdef SIGQUIT
	signal(SIGQUIT, handler);
#endif
#ifdef SIGTERM
	signal(SIGTERM, handler);
#endif
}

private Signal_t handler(sig)
int sig;
{
	/* A signal was caught */

	fatal("caught signal #%d", sig);
}

