case $CONFIG in
'')
	if test ! -f config.sh; then
		ln ../config.sh . || \
		ln ../../config.sh . || \
		ln ../../../config.sh . || \
		(echo "Can't find config.sh."; exit 1)
	fi
	. config.sh
	;;
esac
case "$0" in
*/*) cd `expr X$0 : 'X\(.*\)/'` ;;
esac
echo "Extracting agent/man/mailagent.$manext (with variable substitutions)"
$rm -f mailagent.$manext
$spitshell >mailagent.$manext <<!GROK!THIS!
.TH MAILAGENT $manext ram
''' @(#) Manual page for mailagent's filter -- (c) ram February 1991
'''
''' $Id: mailagent.SH,v 2.9 92/07/14 16:49:11 ram Exp $
'''
'''  Copyright (c) 1991, Raphael Manfredi
'''
'''  You may redistribute only under the terms of the GNU General Public
'''  License as specified in the README file that comes with dist.
'''
''' $Log:	mailagent.SH,v $
''' Revision 2.9  92/07/14  16:49:11  ram
''' 3.0 beta baseline.
''' 
.SH NAME
mailagent \- an automatic mail-processing tool
.SH SYNOPSIS
\fBmailagent\fR [ \fB\-dhilqtV\fR ] [ \fB\-s{umary}\fR ] [ \fB\-f\fI file\fR ]
[ \fB\-e\fI rule\fR ] [ \fB\-c\fI config\fR ] [ \fB\-L\fI loglevel\fR ]
[ \fB\-r\fI rulefile\fR ] [ \fB\-o\fI override\fR ] [ \fImailfile\fR ]
.SH DESCRIPTION
.I Mailagent
allows you to process your mail automatically. Given a set of \fIlex\fR-like
rules, you are able to fill mails to specific folders, forward messages to
a third person, pipe a message to a command or even post the message to a
newsgroup. It is also possible to process messages containing some shell
commands.
The \fImailagent\fR is not usually invoked manually but is rather called via
the \fIfilter\fR program, which is in turn invoked by \fIsendmail\fR.
That means you must have \fIsendmail\fR on your system to use this.
You also must have \fIperl\fR to run the mailagent scripts.
.PP
There is a limited set of options which may be used when you invoke
\fImailagent\fR yourself. You may only specify one special option at a time.
A quick overview of the special options follows, and a complete description
of all the options may be found under the \fBOPTIONS\fR section.
.PP
The \fB\-h\fR option will give you a cryptic usage while \fB\-V\fR prints out
the version number.
More intersting is the \fB\-\q\fR option, which raises a queue processing.
You may also invoke \fImailagent\fR with the \fB\-d\fR option to process
the rule file and see how it is understood. The \fB\-s\fR option, possibly
followed by one or more letters from the {umary} set, will report filtering
statistics, provided this feature was enabled.
.PP
One possible usage of the mailagent is to send patches or whole distributions
to people. You also have a useful feature that will acknowledge the mails
if the sender asks so. This is activated by default rules hardwired in the
program, as explained later on. Providing your own rule file will override
this default behaviour.
'''
''' G e t t i n g   S t a r t e d
'''
.SH "GETTING STARTED"
.PP
First, you need to install a minimum configuration and see how it works. It
would be useless to fully install the program and then discover that it does
not work as advertised...
.PP
To start the installation, you have to set up a \fI~/.mailagent\fR file which is
the main configuration file, and choose the right \fIfilter\fR program.
'''
.SS "Choosing the filter program"
.PP
The distribution comes with two filter programs. One written in shell and one
in C. The shell version might be the one to use if you can receive your mail
on many different platforms where your home directory is NFS-mounted (i.e.
shared among all those platforms). The C version is safer and much faster,
but you need to install it to a fixed loaction.
.PP
On some platforms, \fIsendmail\fR does not correctly reset its UID when
processing mails in its own queue. In that case, you need to get a private
copy of the C filter program and make it setuid on yourself. The filter will
then corretly reset its UID if invoked with an effective UID different from
yours (it may also require the setgid bit to reset GID as well).
If this is indeed the case on your system, make sure you use the
\fIpath\fR configuration variable to set a proper PATH, as the filter will
spawn a perl process with the '-S' option, looking for a \fImailagent\fR
script.
.PP
Even if you do not need to get a setuid copy of the \fIfilter\fR program, it
is wise to set up a proper path: someone might break into your account by
putting a mailagent trojan horse in the appropriate location. Also make sure
the mailagent program is protected against writing, as well as the directory
which holds it, or someone might substitute his own version of the script
and break security. I believe the setuid \fIfilter\fR program is safe, but
overlooking is always possible so please report any security hole to me.
.PP
The \fIfilter\fR script can be found in the \fILib/mailagent\fR directory. It
needs some tailoring so you should copy it into your home directory and edit
it to suit your needs. Comments held in it should be
self explanatory. There is only a small section at the head of the
script which needs to be edited.  You'll have to delete shell comments
in the \fIfilter\fR script by yourself if your shell cannot deal with them.
'''
.SS "Configuring the mailagent"
.PP
You have to copy the \fImailagent.cf\fR file held in the mailagent
sub-directory \fI$privlib\fR (hereafter named Lib)
as a \fI.mailagent\fR in your home directory. Edit it to configure the
whole processing. In particular, you have to choose a spool directory
(hereafter named Spool) and a log directory (hereafter named Log).
.PP
Following is a description of each of those fields, followed by a suggested
value, when applicable. Fields marked as optional may not be present in the
configuration file. Some fields have a close relationship with others, and
that is given too.
.sp
.PD 0
.TP 10
.I agemax
Period after which an entry in the database should be removed (suggested: 1y)
This field is optional, but needed if \fIautoclean\fR is on.
.TP
.I authfile
Remote sending authorizations (not implemented yet).
.TP
.I autoclean
Set to ON (case insensitively), the mailagent will perform automatic cleaning
of the database entries under \fIhash\fR by removing all the items older
than \fIagemax\fR. This is an optional field, omitting it defaults to OFF.
(suggested: OFF, unless you use ONCE, UNIQUE or RECORD commands, or activate
the vacation mode.)
.TP
.I cleanlaps
Cleaning period for database entries. The value of the last clean up is saved
into the context file. This is optional, but needed if \fIautoclean\fR is on.
(suggested: 1M)
.TP
.I comfile
Name of the file containing authorized commands. Needed when PROCESS is used.
(suggested: \$spool/commands).
.TP
.I context
File holding the mailagent context. The context saves some variables which
need to be kept over the life of the process. Needed if auto cleaning is
activated. (suggested: \$spool/context)
.TP
.I distlist
A list of all the available distributions. See the sample held in
\fILib/mailagent/distribs\fR. Needed by PROCESS only. (suggested:
\$spool/distribs)
.TP
.I emergdir
Name of the directory which should be used for dumps, preferably. This is
optional. (suggested: ~/tmp/lost+mail)
.TP
.I hash
The directory used for name hashing by the built-in database used by ONCE,
UNIQUE and RECORD commands. Optional, unless you make use of those commands
or activate auto cleaning. The directory is placed in the spool area.
(suggested: dbr).
.TP
.I home
Defines where the home directory is. This must be accurate.
.TP
.I level
Log level, see below for a definition of available levels (suggested: 9).
.TP
.I log
Name of the log file, put in Log directory. (suggested: agentlog).
.TP
.I logdir
Logging directory. (suggested: ~/var/log).
.TP
.I maildrop
Location of the system mail spool directory. If none is provided, then the
mailagent will use the value determined by Configure.
.TP
.I maxsize
Maximum size in bytes of files before using \fIkit\fR for sending files. This
is used by PROCESS. (suggested: 150000).
.TP
.I name
First name of the user, used by the mailagent when referring to you. This sets
the value of the %U macro.
.TP
.I path
Minimum path to be used by C filter program. To set a specific path
for a machine \fIhost\fR, set up a \fIp_host\fR variable. This will
be \fIprepended\fR to the default \fIPATH\fR variable supplied by other
programs. (suggested: /bin:/usr/bin:/usr/ucb)
.TP
.I plsave
Name of the file used to save the patchlevels for archived distributions.
This is only used by the commands invoked via PROCESS. (suggested:
\$spool/plsave.
.TP
.I proglist
A small description for the available distributions. See the sample
held in \fILib/mailagent/proglist\fR. This is used by PROCESS only.
(suggested: \$spool/proglist)
.TP
.I queue
Queue directory (messages waiting to be processed). Required, of course.
(suggested: \$spool/queue)
.TP
.I rules
The name of the file holding the filtering rules (optional,
suggested: ~/.rules).
.TP
.I seq
File used to compute job numbers (suggested: .seq).
.TP
.I spool
Spool directory, required (suggested: ~/var/mailagent).
.TP
.I statfile
File where statistics should be gathered. If no such file exists, no
statistics will be recorded (suggested: mailagent.st).
.TP
.I track
Set to \fIon\fR (case insensitively), this turns on the \fB\-t\fR option
which tracks all the rule matches and the actions on standard output. This
is optional (suggested: OFF).
.TP
.I timezone
The time zone value for environment variable TZ (optional).
.TP
.I tmpdir
Directory for temporary files. Required (suggested: /tmp).
.TP
.I user
Login name of the user who runs the mailagent. This sets the value of the
%u macro.
.TP
.I vacation
A flag set to ON or OFF to switch the vacation mode accordingly.
.TP
.I vacfile
The name of the file to be sent back in vacation mode (suggested: ~/.vacation).
.TP
.I vacperiod
The minimum time elapsed between two vacation messages to a given address
(suggested: 1d).
.PD
'''
.SS "Available log levels"
.PP
The following log levels can be used while running the mailagent:
.sp
.in +5
.nf
0	No logging
1	Major problems only
2	Failed deliveries
3	Successful deliveries
4	Deferred messages
5	Successful filter actions
6	Unusual but benign incidents
7	Informative messages
8	Non-delivery filter actions
9	Mail reception
12	Debug
19	Verbose
20	Lot more verbose
.fi
.in -5
.sp
'''
.SS "Setting the mail agent"
.PP
Once you have configured the mailagent in a \fI~/.mailagent\fR (where \fI~\fR
stands for your home directory), you must tell \fIsendmail\fR how to invoke it.
This is done by setting a \fI~/.forward\fR file which looks like this:
.sp
.in +5
"| exec /users/ram/mail/filter >>/users/ram/.bak 2>&1"
.in -5
.sp
This will pipe all your mails to the \fIfilter\fR program, redirecting all
unusual messages to \fI~/.bak\fR. A sample filter shell script may be found in
\fILib/mailagent\fR, as well as a C filter program.
.PP
Note that the \fI.forward\fR file only pipes the mail to the \fIfilter\fR
program and does not leave any copy in the mailbox. It is up to you to decide
in the rule file whether you want to trash the mail away or leave it in the
mailbox. If you do not have a rule file (i.e. you left a blank entry in your
\fI~/.mailagent\fR, or you named a non-existent file, or your file is simply
empty), don't worry: the default action is to leave the mail in the mailbox.
'''
.SS "Allowed commands"
.PP
The allowed command file (as specified by the \fIcomfile\fR variable in
your \fI~/.mailagent\fR) contains all the recognized and allowed commands.
The file \fIcommands\fR held in directory \fILib/mailagent\fR should be
copied as-is into your Spool directory.
'''
.SS "Testing your installation"
.PP
Now, assuming you have set a proper \fI~/.mailagent\fR file and edited the
configuration section of the \fIfilter\fR, it is time to test your
installation. Make sure your \fI.forward\fR is world readable and that the
\fIfilter\fR has the execution bits set (there is no reason to make the
\fIfilter\fR world readable).
Set a log-level of 20 and disable vacation mode (the \fIvacation\fR entry in the
\fI~/.mailagent\fR should be OFF). Set the name of the rule file to
\fI/dev/null\fR. You are ready to proceed...
.PP
Send yourself a mail and give the mailagent time to process your mail. The
subject of the message should be 'test' (in fact, anything but 'Command').
You may want to run a "\fItail -f logfile\fR" to see what's happening. At the
end of the processing, the logfile should contain something like the following
(names of temporaries may \-and will\- of course differ; timestamps have been
removed):
.sp
.in +5
.nf
got the right to process mail
building default rules
parsing mail
analyzing mail
in mode 'INITIAL' for ALL
selector 'All', pattern '/^Subject: [Cc]ommand/'
matching '/^Subject: [Cc]ommand/' on 'All' was false
NOTICE no match, leaving in mailbox
XEQ (LEAVE)
starting LEAVE
starting SAVE /usr/spool/mail/ram
LEFT [qm7831] in mailbox
FILTERED [qm7831] from ram (Raphael Manfredi)
mailagent continues
mailagent exits
.fi
.in -5
.sp
.PP
If you do not get that, there is a problem somewhere. Start by looking at
the \fI~/.bak\fR file (or whatever file the \fI.forward\fR uses to redirect
output of the filter). If you see something like:
.sp
.in +5
.nf
FATAL no valid queue directory
DUMPED in ~/mbox.filter
.in -5
.fi
.sp
then it means the \fIqueue\fR parameter in your \fI~/.mailagent\fR does not
point to a valid directory. Your mail has been dumped in an emergency
mailbox.
.PP
The \fI~/.bak\fR file may also contain error messages stating that \fIperl\fR
was not found. In that case, there should be an error message in the logfile:
.sp
.in +5
ERROR mailagent failed, [qm7886] left in queue
.in -5
.sp
In that case, make sure the mail has correctly been queued in a file
\fIqm7886\fR. The queue will be processed again when another mail arrives
or when the \fImailagent\fR is invoked with \fB\-q\fR (however, to avoid
race conditions, only mails which have remained for a while will be processed).
.PP
Queuing of mail also happens when another \fImailagent\fR is running. If the
logfile says:
.sp
.in +5
denied right to process mail
.in -5
.sp
then remove the \fIperl.lock\fR file in the Spool directory. Old lock files
are automatically discarded by the \fImailagent\fR anyway (after one hour).
.PP
If none of these occurs, then maybe \fIsendmail\fR did not process your
\fI~/.forward\fR at all or the file has a syntax error.
Check your mailbox, and if your mail
is in there, your \fI.forward\fR has not been processed. Otherwise, ask your
system administrator to check \fIsendmail\fR's logfile. A correct entry would
appear as (with leading timestamps and syslog stamps removed):
.sp
.in +5
.nf
message-id=<9202041919.AA07882@york.eiffel.com>
from=ram, size=395, class=0, received from local
to="| /york/ram/mail/filter >>/york/ram/.bak 2>&1", delay=00:00:05, stat=Sent
.in -5
.fi
.sp
.PP
If you still cannot find why the mail was not correctly processed, you should
make sure you normally receive mail by removing (or renaming) your
\fI~/.forward\fR and sending yourself another test mail. Also make sure your
home directory is world readable and "executable".
'''
''' O p t i o n s
'''
.SH OPTIONS
There is a limited set of options which may be used when calling the
mailagent directly. Only one special option at a time may be specified.
Invoking the mailagent as \fImailqueue\fR is equivalent to using the
\fB\-l\fR option.
.TP 15
.B \-c\fI file\fR
Specify an alternate configuration file (~ substitution occurs). The default
is \fI~/.mailagent\fR.
.TP
.B \-d
The mailagent parses the rule file, compiles the rules and dumps them on the
standard output. This option is mainly used to check the syntax of the
rule file and make sure the rules are what the user really thinks they are.
.TP
\fB\-e\fI rule\fR
This option lets you specify some rules on the command line, which will
override those specified via the ~/.mailagent, if any. There may be as many
\fB\-e\fR as necessary, all the rules being concatenated together as one
happy array, which is then parsed the same way a rule file is. If only \fBone\fR
rule is given and there is no action specified between {...} braces, then
the whole line is enclosed between braces. Hence saying \fI-e 'SAVE foo'\fR
will be understood as \fI-e '{SAVE foo}'\fR, which will always match and
be exectuted. Using the \fB\-d\fR option in conjunction with this one is a
convenient way to debug a set of rules.
.TP
\fB\-f\fI mailfile\fR
Using \fImailfile\fR as a UNIX-style mailbox (i.e. one where each mail is
preceded by a special From line stating the sender and the date the message
was issued), extract all its messages into the queue and process them as if
they were freshly arrived from the mail delivery subsystem.
.TP
.B \-h
Print out a usage message on the standard error and exit.
.TP
.B \-i
Interactive mode, directs the mailagent to print a copy of all the log messages
on \fIstderr\fR.
.TP
.B \-l
List the mailagent queue. Recently queued mails which are waited for by the
\fIfilter\fR are \fIskipped\fR for about half an hour, to avoid race conditions.
.TP
.B \-L\fI level\fR
Override the log level specified in the configuration file.
.TP
.B \-o\fI overrride\fR
This option lets you override a specific configuration option. The option must
be followed by a valid configuration line, which will be parsed after the
configuration file itself. For instance, the \fI\-L 4\fR option is completely
equivalent to \fI\-o 'level: 4'\fR. Note that any white space must be protected
against shell interpretation by using the appropriate quoting mechanism. There
may be as many \fB\-o\fR options on the command line as necessary.
.TP
.B \-q
Force processing of the mailagent's queue. Only the mails not tagged as
\fIskipped\fR by the \fB\-l\fR option will be processed.
.TP
.B \-r\fI file\fR
Specify an alternate rule file.
.TP
.B \-s {umary}
Build a summary of all the statistics gathered so far. The output can be
controlled by appending one or more letters from the set {umary}. Using
\fB\-summary\fR is a convenient way to get the whole history of the filter
actions. The \fBu\fR modifier will print only used rules. The \fBm\fR will
merge all the statistics at the end while \fBa\fR reports the mode the
filter was in when the command was executed. The \fBr\fR asks for rule-based
statistics and the \fBy\fR is pretty useless and is here only to get a nice
mnemonic option. Note that specifying an option more than once has no effect
whatsoever on the option itself (i.e. you may put three \fbUu\fR and only
one \fBm\fR, but you'll still get the summary!).
.TP
.B \-t
Put the mailagent in a special tracking mode where all the rule matches and
executed actions are printed on the standard output. This is mostly useful
for debugging a rule file. See also the \fItrack\fR parameter in the
configuration file.
.TP
.B \-V
Print version number and exit.
.PP
If you invoke the mailagent without options and without any arguments, the
program waits for a mail on its standard input. If an argument is provided, it
is the name of a file holding one mail to be processed. This is the normal
calling procedure from the filter, the argument being the location of the
queued mail.
''' -----------------
'''
''' D e f a u l t   R u l e s
'''
.SH "USING THE DEFAULT RULES"
If you do not want to use the filtering feature of the mailagent, then the
default built-in rules will be used. Those are really simple: all the mails
are left in your mailbox and mails with a line "Subject: Command" anywhere in
the message will be processed. Commands are looked for on lines starting with
"@SH". The remaining of the line is then given to a shell for execution.
.PP
Available commands are read from a file (entry \fIcomfile\fR in your
configuration file), one command name per line. Only those listed there will
be executed, others will produce an error message. The mailagent traps
the exit status and will send an error report if a command fails (provided
that the command does not issue a message by itself, in which case it
should return a zero exit status).
.PP
If you do not want to use the default rules, you may skip the remaining of this
section.
'''
.SS "Configuring help"
.PP
The help text the mailagent will send to people must be copied from
\fILib/mailagent/agenthelp\fR into your own spool directory, as specified in your
\fI~/.mailagent\fR. Two macros may be used:
.TP 10
.I =DEST=
This will be expanded to the sender's address (the one who sent you
the mail currently processed by the mailagent).
.TP
.I =MAXSIZE=
This stands for the maximum size set before \fIkit\fR is used to send
files back (parameter \fImaxsize\fR in your \fI~/.mailagent\fR file\fR).
.PP
You may use the default help file or design one that will give even more
details to the poor user.
'''
.SS "Distribution files"
.PP
The two files \fIproglist\fR and \fIdistribs\fR held in \fILib/mailagent\fR
describe the distributions your mailagent will be able to distribute.
The samples given show the expected syntax. In order to clarify things,
here is what the format should be:
.PP
File \fIproglist\fR contains a small description for programs. The name
of the program appears after a single star. It is followed by lines in
free format. An optional three-dashes line separates each program's
description. Note that a leading tab will be added to each line
of description.
.PP
The \fIdistribs\fR file holds lines of the following form:
.sp
.in +5
\fIprogname version path archived compressed patches\fR
.in -5
.sp
where:
.TP 10
.I progname
is the program name (the same as the one mentioned in \fIproglist\fR).
.TP
.I version
is the current version number. If none, a three-dashed line may be used.
.TP
.I path
is the path where the distribution is stored. The ~ will be expanded into
your home directory. Note that if the distribution is stored in archived
form, the path name is the one of the archive without the ending
extension (which may be \fI.cpio.Z\fR or \fI.tar.Z\fR).
.TP
.I archived
is either \fIy\fR or \fIn\fR depending on whether the distribution is
archived or not.
.TP
.I compressed
is either \fIy\fR or \fIn\fR depending on whether the distribution is
compressed or not. This could be guessed from the extension's name, but
we must think of file systems with short names.
.TP
.I patches
is \fIy\fR or \fIn\fR depending on whether the distribution is
maintained or not by you. If you put a \fIp\fR, this means official
patches are available, although you do not maintain the distribution.
Finally, an \fIo\fR means that this is an old version, where only patches
are available, but maildist will not work. In that case, assuming the
version number is \fI1.0\fR, old patches are expected in a \fIbugs-1.0\fR
directory.
.PP
You may include comments in both files: all lines starting with a leading
# will be ignored.
'''
.SS "Testing your mail agent"
.PP
It is now time to make sure your mailagent works. Send yourself the following
mail:
.sp
.in +5
.nf
Subject: Command
@SH mailhelp
.fi
.in -5
.sp
You should receive back a mail from yourself with the subject set to:
"How to use my mailagent". If you don't, check the file \fI~/.bak\fR
(or whatever file you set in your \fI.forward\fR). If it is empty, look
at the log file. If the log file is not empty, then perhaps the mail
has been queued. Check the \fIsendmail\fR queue. Also make sure that
you removed the '#' comments in the \fIfilter\fR script. On some systems,
they cause some trouble. If you are using the C filter, maybe your sendmail
is broken and you need to make your own setuid copy (or perl might complain
that you have a kernel bug, etc...).
.PP
If you have done everything right but it still does not work properly,
increase log level to 20 and resend your command mail. Then check the
log file. The diagnosis should be easier.
.PP
Once this works, you should check your \fIdistribs\fR and \fIproglist\fR
files by sending yourself the following mail:
.sp
.in +5
.nf
Subject: Command
@SH maillist
.fi
.in -5
.sp
If the list you have in return is incorrect, then your distribution files
are wrongly written. If you do not get the list, there is a problem with
your mailagent's configuration. Retry with a log level set to 20 and look
at the issued log messages in your Log directory. Make sure that the file
listed in the \fIplsave\fR entry of your \fI~/.mailagent\fR is correctly
updated after a \fImaillist\fR has been run.
'''
''' F i l t e r i n g   R u l e s
'''
.SH "USING THE FILTER"
The \fImailagent\fR can also be used as a filter: mail is parsed and some
actions are taken based on simple \fIlex\fR-like rules. Actions range from
a simple saving in a folder, a forwarding to another person, or even spawning
of a shell command. Before going further, here is a small example of a valid
rule file:
.sp
.in +5
.nf
From: root { FORWARD postmaster };
To: gue@eiffel.fr { POST mail.gue };
Subject: /metaconfig/ { SAVE dist };
.fi
.in -5
.sp
There are three distinct rules. Rules are applied in sequence, until one
matches (so the order is important). Any mail coming from \fIroot\fR will be
forwarded to user \fIpostmaster\fR. A mail addressed to \fIgue@eiffel.fr\fR is
a mail coming from a mailing list. The mail is posted on a local newsgroup
\fImail.gue\fR. Mails whose subject contains the word "metaconfig" will be
saved in a folder \fIdist\fR for delayed reading and will not appear in the
main mailbox. If no rule matched, the mail is left in the mailbox.
'''
.SS "Syntax of the rule file"
.PP
Here is a non-formal description of the rule file. Parsing of the file is done
lexically, hence the choice of non-ambiguous tokens like '{' or ';' which are
easily parsed. This introduces some limitations which are silently applied:
for instance, no '{' may be used as part of an address.
.PP
Comments are introduced by a leading '#' , which must be on the left margin.
Unlike shell comments, a '#' which is not left justified will not be understood
as a comment. However, spaces or tabs are allowed in front of '#'.
.PP
All the statements in the rule file must end with a ';'. There are mainly
four parts in each line. A list of comma separated modes, between '<' and '>',
which give the set of modes in which the rule applies. The special mode
ALL will match everything. The filter begins in the mode INITIAL. Omitting the
mode defaults to "<ALL>".
.PP
Then comes a list of selectors. Those selectors must be space separated and end
with ':'. They represent the names of header fields which must be looked at
by the forthcoming pattern. An empty selector list defaults to "Subject:".
Special selectors "All:", "Body:" and "Head:" apply to the whole message,
its body or its header. A commonly used selector list is "To Cc:" which tests
the recipient fields of the header.
.PP
The selector is then followed by a pattern within '/' or by a single name.
In order to ease the writing of the rules, the semantic of a single name varies
depending on the selector used. For the special selector "From:", "To:", "Cc:"
and "Apparently-To:", a single name is understood as a match on the \fIlogin
name\fR of the address. Note that if no "To:" field is present in the header,
one will be forged from the "Apparently-To:" for the purpose of filtering only
(i.e. no physical modification on the header is done). If the login name of
the address is a full name of the form First.Last, only the last name is
kept, and is lower-cased. If only a single name is given, only shell
metacharacters * and ? are allowed, as well as intervals [].
.PP
If a single word
is used for non-special selectors, the same rules apply but the pattern is
anchored at the beginning and the end for an exact match. With a pattern
starting with '/', any regular expression understood by \fIperl\fR may be
used and your pattern will not be modified in any way. The other special
selector "Newsgroups:" works as "To:", excepted that newsgroups names are
expected and a match is attempted on every item in the list. Every pattern
match on a single name for an address-type field (i.e. "Newsgroups:" excluded),
are made in case-insensitive mode.
.PP
There is also a little magic involved when matching on an address field. Namely,
if the pattern is not a single word and is \fIanchored at the beginning\fR,
then only the address part of the field will be kept. For instance, if we
have a From: field whose value is \fIRaphael Manfredi <ram@eiffel.com>\fR, then
the pattern \fI/Raphael/\fR would match, but not \fI/^Raphael/\fR. Instead,
\fI/^ram@.*$/\fR would match, but this is more easily done with a single word
pattern \fIram\fR, for it only focuses on the login name of the address and
would also match if the address was written as \fIeiffel.com!ram\fR.
.PP
This may sound a little complex, but this design is meant to make things
easier for the user. Here are some other examples:
.sp
.in +5
.nf
# Match 'ram@eiffel.com', as well as 'ram@educ.emse.fr'.
From: ram

# Match 'root@eiffel.com', 'ram' but not 'ribbon@eiffel.com'
From: r[oa]*

# Match 'gue@eiffel.fr' but not 'algue@eiffel.fr'
To Cc: gue@eiffel.fr

# This will match 'gue@eiffel.fr' as well as 'algue@eiffel.com'
To Cc: /gue@eiffel/

# Match 'comp.lang.perl' but not 'comp.lang.perl.poetry' (?)
Newsgroups: comp.lang.perl
.fi
.in -5
.sp
When attempting a match on "To:", "Cc:" or "Apparently-To:", a
list of addresses separated by a comma is expected, whereas only one
address is expected after "From:". If you omit the pattern, it will be
understood as * (recall that a single word uses shell meta-characters), which
will match anything.
.PP
Then comes the action to be taken when a match occurs. There are only a
limited set of valid actions which will be described soon in detail. The
action is enclosed in curly braces '{' and '}' and actions are separated or
terminated (depending on your taste) by a ';'. Action's name are spelled in
upper-case for readability, but case is irrelevant.
.PP
Note that a rule should
be ended by a single ';' after the last '}'. It is possible to omit this
final ';', but that single token is the resynchronizing point for error
recovery. One could argue however that there should be no syntax error, and
thus the ';' ought to be safely omitted. Whenever in doubt, check your rule
file with the \fB\-d\fR option.
.PP
Here is a prototypical rule:
.sp
.in +5
.nf
<ROOT> From: /^\\\\w+@eiffel.com$/ { SAVE eiffel };
.fi
.in -5
.sp
That rule will only be taken into account when the filter is in the mode ROOT
(recall that the processing starts in mode INITIAL; use BEGIN to change the
mode, as in \fIlex\fR). So in mode ROOT, anything which comes from a user
located in the \fIeiffel.com\fR site is saved in folder \fIeiffel\fR for
deferred reading. The mail will not appear in the mailbox.
.PP
It is possible to have more than one selection for a rule. Identical selectors
are logically \fIor\fR'ed while different ones are \fIand\fR'ed. The selections
are comma separated. For instance,
.sp
.in +5
.nf
From: root, To: ram, From: ram, Subject: /\\\\btest\\\\b/ { DELETE };
.fi
.in -5
.sp
will delete a mail from \fIroot\fR or \fIram\fR if it is sent to \fIram\fR
and has the word \fItest\fR in its subject. It is also possible to write the
previous rule as:
.sp
.in +5
.nf
From: root, ram, To: ram, Subject: /\\\\btest\\\\b/ { DELETE };
.fi
.in -5
.sp
because if no selector is given, the previous one is used (with the first
selector being "Subject:" by default).
.PP
Anywhere in the rule file, it is possible to define some variables. The list
of recognized variables is given later. For now, let's say that \fImaildir\fR
is the default folder directory. This variable is used by the SAVE command
when the argument is not an absolute path. Setting
.sp
.in +5
maildir = ~/mail;
.in -5
.sp
will direct the filter to use \fI~/mail\fR as the folder directory (default is
\fI~/Mail\fR). Note the ~ substitution and the final ';'. It is not possible
(currently) to modify the environment by setting PATH for instance.
.PP
Finally, there is a special construct to load patterns from a file. A pattern
enclosed in double quotes means that the patterns to be applied should be
taken from the specified file. The file is expected to be in the directory
\fImailfilter\fR if it is not an absolute path (~ substitution occurs). If
the variable is not set \fImaildir\fR will be used. If by chance (!)
\fImaildir\fR is not set either, the home directory is used. The file should
contain one pattern per line, shell comments (#) being allowed at the beginning
of each line.
.PP
An action may be followed by other rules. Hence the following is perfectly
valid:
.sp
.in +5
.nf
From:
	ram		{ SAVE ram }
	/plc/i		{ SAVE plc }
	root		{ SAVE ~/admin }
	/xyz/		{ DELETE }
	"users"		{ LEAVE }
	;
.fi
.in -5
.sp
Note the use of the file inclusion: all the users listed in file \fIusers\fR
will have their mail left in the system mailbox. The usual rules apply for
these loaded patterns.
'''
.SS "Variables"
.PP
The mailagent supports user-defined variables, which are globals. They are
set via the ASSIGN command and referred to with the %# macro. Assuming we
set a variable \fIhost\fR, then %#\fIhost\fR would be replaced by the actual
value of the variable. This enables some variable propagation across the rules.
.PP
For example, let's say the user receives cron outputs from various machines
and wishes to save them on a per-machine basis, differentiating between
daily outputs and weekly ones. Here is a solution:
.sp
.in +5
.nf
Subject: /output for host (\\\\w+)/	{ ASSIGN host %1; REJECT };
Subject: /^Daily output/	{ SAVE %#host/daily.%D };
Subject: /^Weekly output/	{ SAVE %#host/weekly.%m-%d };
.fi
.in -5
.sp
Besides variable interpolation via the %# escape, it is also possible to
perform substitutions and translations on the content of a variable (or a
backreference, i.e. a number between 1 and 99). The two commands SUBST and
TR will respectively perform in-place substitutions and translations. In that
case however, the name of the variable must be preceded by a single #. This
differentiates the backreference \fI1\fR from the variable \fI#1\fR, although
\fI1\fR is a funny name for a variable. The need for # also prevents the
common mistake of writing %#, as the mailagent will loudly complain if the
first parameter of SUBST or TR is not a digit between 1 and 99 or does not
start with a #.
.PP
Here are some actions to cannonicalize the host name into lower case and
strip down the domain name, if any:
.sp
.in +5
.nf
{ TR #host /A-Z/a-z/; SUBST #host /^([^.]*)\\\\..*/\$1/ };
.fi
.in -5
.sp
Those actions are directly translated into their \fIperl\fR equivalent, and
any error in the specification of the regular expression will be reported.
'''
.SS "Regular expressions"
.PP
All the regular expressions follow the V8 syntax, as in \fIperl\fR, with all
the \fIperl\fR extensions. If a bracketing construct (...) is used inside a
rule, then the %\fIdigit\fR macro matches the \fIdigit\fR's substring held
inside the bracket. All those backreferences are memorized on a per-rule basis,
numbered from left to right. However, great care must be taken when using
a backreference in multiply present selectors, as all the matches will be
performed up-to the first match, and backreferences are computed on the fly
while doing pattern matching.
.PP
For instance:
.in +5
.sp
.nf
To: /(.*)/, Subject: /Output from host (\\\\w+)/ { ASSIGN to %1; SAVE %2 };
.fi
.sp
.in -5
will save the To: field in variable 'to' and save the mail in a folder derived
from the host name specified in the subject. However, if we say:
.in +5
.sp
.nf
Subject: /host (\\\\w+)/, /from (\\\\w+)/ { ASSIGN match %1 };
.fi
.sp
.in -5
then there will be only one backreference set, and it will come from the first
pattern matching if it succeeds, or from the second. Should the second or
the first pattern have no bracketing construct and still match, then the
backreference would not be recorded at all, which means the following is
probably not what you want:
.in +5
.sp
.nf
Subject: /from/, /host (\\\\w+)/, To: /(.*)/ { SAVE %1; REJECT };
.fi
.sp
.in -5
as if the /from/ pattern matches then /host (\\\\w+)/ will not be checked
(identical selectors are \fIor\fR'ed and that is optimized), then %1 would refer
to the To: field whereas if /host (\\\\w+)/ matches, then %1 will be the host
name.
.PP
There is also a special macro %&, which lists (it's a comma separated list)
all the selectors specified via a regular expression which indeed matched.
For instance:
.sp
.in +5
.nf
Re.*: /york/		{ ASSIGN which %& };
.sp
.in -5
.fi
would assign to \fIwhich\fR the list of all the fields matching the /Re.*/
pattern which contained 'york', be it a Received: field or a Resent-From:
field (as both match the selector specification). Assuming both those fields
contained the word \fIyork\fR, the value of %& would be 'Received,Resent-From;'
(the fields are alphabetically sorted).
.PP
Should you have more than one such specified selector within a single rule,
then it might be worth knowing that all the set of matching selectors are
recorded within %&, each set terminated with a ';'.
'''
.SS "Available actions"
.PP
The following actions are available as filtering commands. Case is irrelevant
although the recommended style is to spell them upper-cased:
.TP 10
ABORT
Abort application of filtering rules immediately.
.TP
ANNOTATE [-d] \fIfield value\fR
Annotate message by adding \fIfield\fR into the mail header, whith the
supplied \fIvalue\fR. This is like the MH command \fIanno\fR, but the
annotation is performed at the end of the header, whereas MH does it at the
top. Normally, an extra \fIfield\fR is added, with the current date as
field value. This can be suppressed by using the \fB\-d\fR option. If
\fIvalue\fR is ommitted, only the date field is generated (hence it is an
error to use the \fB\-d\fR option without supplying a \fIvalue\fR). As with
all the commands which alter the header, a RESYNC is necessary for the filter
part to actually see the new header.
.TP
ASSIGN \fIvar value\fR
Assign the value to the user-defined variable \fIvar\fR, which may further be
accessed as \fI%#var\fR for macro substitution or \fI#var\fR in the TR and
SUBST commands in place of the variable name. Note that there is no leading
\fI#\fR in front of the variable name.
.TP
BACK \fIcommand\fR
Execute \fIcommand\fR and take its output as new actions to be performed on
the mail (hence performing something analogous to \fI\`command\`\fR in shell).
If there is no output, nothing is done. BACK commands can be nested, although
this may lead to surprises this manpage will not disclose (but I assure you
it will be funny, assuming we have the same sense of humour... :-). Note that
both the standard output and the standard error from the command are used.
If the command fails, the output is mailed back to the user and no action is
performed. Furthermore, normal feedback does not occur here: any output from
the command is taken as filter actions, which means the semantics of PASS,
for instance, is changed: we do not take a body back but commands.
.TP
BEGIN \fIstate\fR
Enter a new state. An explicit REJECT or RESTART is necessary to abort the
processing of the current rule. The processing begins in state INITIAL.
.TP
BOUNCE \fIaddress(es)
Bounce the message to the specified address(es) and acts as if a save has
been done. The only difference with FORWARD is that no Resent-like lines are
added to the header. If an address is specified in double quotes, it is taken
as the name of a file to be loaded to get addresses (one address per line, shell
comments (#) allowed). The file name resolving is the same as the one used
for pattern loading.
.TP
DELETE
Delete the current message. Actually, this does not do anything, it just marks
the mail as saved. If no further action involving saving is done, then the
mail will never show up in the mailbox.
.TP
FEED \fIprogram\fR
Feed the whole message to a program and get the output back as the new
message. The header structure used by the rules is not updated: an explicit
RESYNC is necessary. Hence the program appears as a filter for the whole
message.
.TP
FORWARD \fIaddress(es)\fR
Forward mail to the specified address(es). This acts as if a save had been
done, in order to avoid the DELETE. Usually when you forward a mail, you do not
wish to keep it. The command adds Resent-like lines in the header. As for
BOUNCE, file inclusion is possible (i.e. use an address \fI"forward_list"\fR
to forward a mail to all the users listed in the file \fIforward_list\fR).
.TP
GIVE \fIprogram\fR
Give the body of the message to the specified program by feeding its standard
input. Any output is mailed to the user who runs the \fImailagent\fR.
.TP
KEEP \fIheader_fields_list\fR
Keeps only the corresponding lines in the header of the mail. For instance,
a "KEEP From To Cc Subject" will keep only the principal fields from the
mail message. This is suitable for archving mailing lists messages.
You may add a ':' after each header field name if you wish, but that is not
strictly necessary.
.TP
LEAVE
Leave incoming mail in the system mailbox. This is the default action if no
rule matched or if no saving occurred.
.TP
MESSAGE \fIfile\fR
Send message \fIfile\fR back to the sender of the message (as derived from the
header of the message). The text of the message is run through the macro
substitution mechanism (described later on).
.TP
NOP
No operation. If this seems a bit odd, think of it in terms of a ONCE command.
.TP
NOTIFY \fIaddress\fR \fIfile\fR
Send a notification message \fIfile\fR to a given address. The text of the
message is run through the macro substitution mechanism (described later on).
.TP
ONCE \fI(name, tag, period) command\fR
Execute the specified filter command once per \fIperiod\fR. The \fIname\fR and
\fItag\fR fields are used to record timestamps of the last ONCE command.
More on this later.
.TP
PASS \fIprogram\fR
Feed the body of the message to the specified program and get a new body
back from the output of the program.
.TP
PIPE \fIprogram\fR
Pipe the whole message to the specified program, but do not get anything
back. Any output is mailed to the user who runs the \fImailagent\fR.
.TP
POST [-l] \fInewsgroup(s)\fR
Post the message to the specified newsgroup(s) after having cleaned-up the
header (by removing mail-related fields like Received: or To:). This acts
as a saving. If the first name is \fB\-l\fR as in "POST -l comp.mail.mh",
then a "Distribution: local" header is added to force a local delivery.
Otherwise, the default \fIinews\fR distribution will be used (world,
usually).
.TP
PROCESS
Run the mailagent processing which looks for @SH commands and executes
them. This was described before in the section dealing with default rules.
The action associated by default to a mail having [Cc]ommand as its
subject is PROCESS.
.TP
PURIFY \fIprogram\fR
Feed the header into a program and get new header back. No RESYNC is done.
This may be used to indeed purify the header by removing all the verbose
stuff added by so many mail transport agents (X-400 like lines for instance).
.TP
QUEUE
Queue mail again. A successful queuing counts as if mail had been saved.
Mail queued that way will not be processed during the next 30 minutes. Note
that unless the mailagent is invoked on a regular basis by \fIcron\fR, the
mail will remain in the queue until another mail arrives.
.TP
RECORD [-a] [-r]
Record message in the history and enters mode _SEEN_ if the messsage was
already present there. If the message is recorded for the first time, processing
continues normally. Otherwise a REJECT is performed. If \fB\-r\fR was used,
a RESTART is used instead whilst \fB\-a\fR would run an ABORT. See also UNIQUE.
.TP
REJECT
Abort execution of current action, and continue matching.
.TP
RESTART
Abort execution of current action and restart the matching process from the
beginning. To avoid loops, each rule may be walked through once in a given
mode.
.TP
RESYNC
Resynchronize header used for matching with the header of the mail. This is
probably useful only when a FEED command was run.
.TP
RUN \fIprogram\fR
Run the specified program and mail any output to the user who runs the
\fImailagent\fR.
.TP
SAVE \fIfolder\fR
Save message in the specified folder.
.TP
SELECT (\fIstart .. end\fR) \fIcommand\fR
Execute the \fIcommand\fR only within the time selection period specified.
Dates can be sepecified in a wide range of formats. The output of the
\fIdate\fR(1) command is an example of a valid specification. If the date,
the year or the month is missing, then the current one is substituted in place
of it. The following dates are valid specifications: '10:04:25', 'now'
,'April 1 1992', 'Dec 25', 'July 14 1789, 07:40' (er... it's valid according
to the grammar, but it's before the Epoch so it does not mean anything). Other
fancy dates like 'last month - 5 minutes' or '3 weeks ago' are also enabled.
(Isn't that great to have a \fBreal\fR parser? The filtering rules could have
been more elaborated if only I had known about this Berkley \fIyacc\fR
producing a \fIperl\fR parser...)
.TP
SPLIT [-deiw] \fIfolder\fR
Split a mail in digest format into the specified folder. If no folder is
specified, each digest item is queued and will be analyzed as a single mail
by itself. The \fB\-d\fR option deletes the digest header. The \fB\-i\fR
option means split is done in-place and the original mail is discarded. All the
options may be used simultaneously provided they are stuck together at the
beginning (option parsing being really rudimentary).
If the mail is not in digest format and a folder is specified, then it is saved
in that folder. Otherwise, the SPLIT action fails and nothing occurs (the
filter continues its processing though). The SPLIT command will correctly
burst RFC-934 digest messages and will try to do its best otherwise. If the
digest was not RFC-934 compliant and there is a chance SPLIT might have
produced something incorrect, then the original message is also saved
if \fB\-i\fR, otherwise it is not tagged as saved (so that the default LEAVE
command may apply). The \fB\-w\fR (watch) requests special care and will detect
every non RFC-934 digest, even when the non-compliance is otherwise harmless;
furthermore, any trailing garbage longer that 100 bytes will be saved as a
digest item by itself. Finally, the \fB\-e\fR option will discard the digest
header only if its body is empty (i.e. the moderator did not include any
leading comment).
.TP
STORE \fIfolder\fR
Save message in the specified folder and leave a copy in the system mailbox.
.TP
STRIP \fIheader_fields_list\fR
Remove the corresponding lines in the header of the mail. For instance,
a "STRIP Newsgroups Apparently-To" will remove the appropriate lines to wipe
out any Newsgroups: or Apparently-To: header. You may add a ':' after each
header field name if you wish, but that is not strictly necessary.
.TP
SUBST \fIvar expression\fR
Substitutes the expression on the specified user-defined variable (name
starting with a #) or backreference (digit). For instance, 'SUBST #foo /w/y/g'
would substitute in user-defined variable \fIfoo\fR all the \fIw\fR by \fIy\fR.
See also ASSIGN and TR.
.TP
TR \fIvar translation\fR
Perform the translation on the specified variable or backreference. For
instance, 'TR 1 /A-Z/a-z/' would cannonicalize content of reference 1 into
lowercase. See also ASSIGN and SUBST.
.TP
UNIQUE [-a] [-r]
Record message in the history and tag message as saved if it was
already present there. If the message is recorded for the first time, processing
continues normally. Otherwise a REJECT is performed. If \fB\-r\fR was used,
a RESTART is used instead whilst \fB\-a\fR would run an ABORT. See also RECORD.
For instance, to remove duplicate messages from mailing lists, run a
UNIQUE -a before saving the mail.
.TP
VACATION \fIon/off\fR
Allow or disallow a vacation message. When vacation mode is turned on via the
configuration file, a message is sent whenever the user receives a mail
meeting some requirements, as explained under the section \fBVACATION MODE\fR.
One of the condition is that the vacation flag modified by this command be
true. This makes it easy to disallow vacation messages, ever, to a group
of people for instance.
.TP
WRITE \fIfolder\fR
Write the message in the specified folder, removing any pre-existing folder
with the same name. Hence, successive WRITE commands will overwrite the
previous one. This is useful to store output of system commands ran by
\fIcron\fR.
'''
.SS "Macros substitutions"
.PP
All the commands go through a macro substitution mechanism before being
executed. The following macros are available:
.sp
.PD 0
.TP 10
%%
A real percent sign
.TP
%D
Day of the week (0-6)
.TP
%H
Host name (name of the machine on which the \fImailagent\fR runs)
.TP
%L
Length of the body part, in bytes
.TP
%N
Full name of the sender (login name if none)
.TP
%R
Subject of the original message with leading Re: suppressed
.TP
%S
Re: subject of original message
.TP
%T
Time of the last modification on mailed file (commands MESSAGE and NOTIFY)
.TP
%U
Full name of the user
.TP
%_
A white space (useful to put white spaces in single patterns)
.TP
%&
List of selectors which incurred match (among those specified via a regular
expression such as 'X-*: /foo/i'. If we find the \fIfoo\fR substring in the
X-Mailer: header line, then %& will be set to this value). Values in the list
are comma separated.
.TP
%~
A null character, wiped out from the resulting string.
.TP
%\fIdigit\fR
Value of the corresponding back reference from the last match.
.TP
%#\fIvar\fR
Value of user-defined variable \fIvar\fR
.TP
%d
Day of the month (01-31)
.TP
%f
Contents of the "From:" line, something like %N <%r> or %r (%N) depending
on how the mailer is configured.
.TP
%h
Hour of the day (00-23)
.TP
%i
Message ID, if available (otherwise, this is a null string)
.TP
%l
Number of lines in the message
.TP
%m
Month of the year (01-12)
.TP
%n
Lower-case login name of sender
.TP
%o
Organization (where \fImailagent\fR runs)
.TP
%r
Return address of message
.TP
%s
Subject of original message
.TP
%t
Current hour and minute (in HH:MM format)
.TP
%u
Login name of the user
.TP
%y
Year (last two digits)
.TP
%[To]
Value of the header field (here To:)
.PD
'''
.SS "Using once commands"
.PP
The ONCE constructs lets you specify a given command to be run once every
period (day, week...). The command is identified by a \fIname\fR and a
\fItag\fR, the combination of the two being unique. Why not just a single
identifier? Well, that would be fine, but assume you want to send a message
in reply to someone once every week. You could use the e-mail address of the
person as the command identifier. But what if you also want to send another
message to the same address, this time once a month?
.PP
Here is a prototypical usage of a ONCE, which acts like the vacation program,
excepted that it sends a reply only once a day for a given address:
.sp
.in +5
{ ONCE (%r, message, 1d) MESSAGE ~/.message };
.in -5
.sp
This relies on the macro substitution mechanism to send only once a day the
message held in \fI~/.message\fR. Do not use the tag \fIvacation\fR, unless
you know what you are doing: this is the tag used internally by the mailagent
in vacation mode. Recall that no selector nor pattern is understood as
"Subject: *", hence the rule is always executed because that pattern always
matches.
.PP
The timestamps associated with each commands are kept in files under the Hash
directory. The name is used as a hashing key to compute the name of the file
(the two first letters are used). Inside the file, timestamps are sorted by
name, then by tag. Of course, you could say (inverting tag and name):
.sp
.in +5
{ ONCE (message, %r, 1d) MESSAGE ~/.message };
.in -5
.sp
but that would be likely to be less efficient, as the first hashing would be
done on a fixed word, hence all the timestamps would be located in the file
\fIHash/m/e\fR (where \fIHash\fR is the name of your hashing directory, which
is the \fIhash\fR parameter in the configuration file).
'''
.SS "Specifying a period"
.PP
The period parameter of the ONCE commands or the \fIvacperiod\fR parameter
of your configuration file has the following format: a number followed by a
modifier. The modifier is an atomic period like a day or a week, the number
is the number of atomic periods the final period should be equal to. The
available modifiers are:
.sp
.PD 0
.TP 10
m
minute
.TP
h
hour (60 minutes)
.TP
d
day (24 hours)
.TP
w
week (7 days)
.TP
M
month (30 days)
.TP
y
year (365 days)
.PD
.PP
All the periods are converted internally in seconds, although you do not
really care... Examples of valid periods range from "1m" to "136y" on a 32 bits
machine (why ?).
'''
.SS "Timeouts"
.PP
In order to avoid having a \fImailagent\fR waiting for a command forever, a
maximum execution time of one hour is allowed. Past that amount of time, the
child is sent a SIGTERM signal. If it does not die within the next 30 seconds,
a SIGKILL is sent. Output from the program, if any so far, is mailed back to
the user.
'''
.SS "Avoiding loops"
.PP
The \fImailagent\fR leaves a "X-Filter:" header on each filtered message,
which in turn is used to detect loops. If a message already filtered is
to be processed, the \fImailagent\fR enters a special state _SEEN_. This
state is special in the sense it is built-in, it is not matched by ALL, and
some actions are not made available, namely: BACK, BOUNCE, FEED, FORWARD, GIVE,
NOTIFY, PASS, PIPE, POST, PURIFY, QUEUE and RUN. Also note that although the
ONCE and SELECT constructs are enabled, they will not let you execute
disallowed commands.
.PP
The _SEEN_ mode makes it easy to deal with mails which loop because of an
alias loop you have no control on. If no action is found in the _SEEN_ mode,
the mail is left in the mailbox, as usual. Moreover, if no saving is done,
a LEAVE is executed. This is the normal behavior.
'''
.SS "Message files"
.PP
The text of the message to be sent back (for MESSAGE or NOTIFY) is read from
a file and passed through the macro substitution mechanism. The special macro
\fI%T\fR is set to the date of last modification made on that file. The format
is \fImonth/day\fR, and the year is added before the month only if it differs
from the current year.
.PP
At the head of the message, you may put header lines. Those lines will
overwrite the default supplied lines. That may be useful to change the default
subject or add some additional fields like the name of your organization.
The end of your header is given by the first blank line encountered.
If the top of the message you wish to send looks like a mail header, you may
protect it by adding a blank line at the very top of the file. This dummy line
will be removed from the message and the whole file will be sent as a body
part.
.PP
Here is an example of a vacation file. We add a carbon copy as well as the
name of our organization in the header:
.sp
.in +5
.nf
Organization: %o
Cc: ram

[Last revision made on %T]

Dear %N:

I've received your mail regarding "%R".
It will be read as soon as I come back from vacation.

Sincerely,
--
%U <%u@%H>
.fi
.in -5
.sp
.SH "VACATION MODE"
.PP
When it's time to take some vacation, it is possible to set up the mailagent
in vacation mode. Every \fIvacperiod\fR, the message \fIvacfile\fR will be
sent back to the user (with macros substitutions) if the user is explicitely
listed in the \fITo\fR or \fICc\fR field and if the sender is not a special
user (\fIroot\fR, \fIuucp\fR, \fInews\fR, \fIdaemon\fR, \fIpostmaster\fR,
\fInewsmaster\fR, \fIusenet\fR, \fIMAILER-DAEMON\fR or \fInobody\fR).
Matches are done in a case insentive manner, so \fIMailer-Daemon\fR will also
be recognized as a special user.
.PP
Internally, the mailagent uses a ONCE
command tagged \fI(%r, vacation, \$vacperiod)\fR. This implies you must not
use the \fIvacation\fR tag in your own ONCE commands, unless you know what
you are doing.
.PP
Besides, the vacation message is sent only if no "VACATION off" commands were
issued, or if another "VACATION on" overwrote the previous one. Note that
whether a rule matched or not is irrelevant to the algorithm. By default, of
course, the vacation message is allowed when the \fIvacation\fR configuration
parameter is set to \fIon\fR.
.PP
If you are not pleased by the fact that a vacation message is sent to people
who addressed you a carbon copy only, then you may write at the top of your
rule file:
.sp
.in +5
.nf
Cc: ram  { VACATION off; REJECT };
.fi
.in -5
.sp
Of course, you have to susbstitute your own login name in place of \fIram\fR.
You cannot use the same scheme to allow vacation messages to special
users like \fIroot\fR, because the test for "specialness" occurs after the
vacation mode flag. This is construed as a feature as it prevents stupid
mistakes, like using \fIr*\fR instead of \fIram\fR in the previous rule.
.SH VARIABLES
The following variables are paid attention to: they may come from the
environment or be set in the rule file:
.TP 10
.I mailfilter
indicates where loaded patterns are to be looked for, if the name of the
file is not fully qualified. If it is not set, \fImaildir\fR will be used
instead. If \fImaildir\fR is not set either, the home directory is used.
.TP
.I maildir
is the location of your mail folders. Any relative path is understood as
starting from \fImaildir\fR. If it is not set, \fI~/Mail\fR is used.
.SH AUTOMATIC ACKNOWLEDGMENTS
Anywhere in the mail, there can be an @RR left-justified line which will
send back an acknowledgment to the sender of the mail. The @RR may optionally
be followed by an address, in which case the acknowledgment will be sent
to that address instead.
In fact (but let's keep that a secret), this is a way for me to be able to
see who runs my mailagent program and who doesn't...
.PP
The \fIsendmail\fR program usually implements such a feature via a
Return-Receipt-To: header line, which sends the whole header back upon
successful delivery. However, this is not implemented on all mail transport
agents, and @RR is a good alternative :-).
.SH EXAMPLES
Here are some examples of rule files. First, if you do not specify a rule
file or if it is empty, the following built-in rule applies:
.sp
.in +5
.nf
All: /^Subject: [Cc]ommand/ { LEAVE; PROCESS };
.fi
.in -5
.sp
Every mail is left in the mailbox. Besides, mail with "Subject: Command"
anywhere in the message are processed.
.PP
The following rule file is the one I am currently using:
.sp
.in +5
.nf
maildir = ~/mail;

All: /^Subject: [Cc]ommand/	{ SAVE cmds; PROCESS };

To: gue@eiffel.fr		{ POST -l mail.gue };
Apparently-To: ram,
Newsgroups: mail.gue		{ BOUNCE gue@eiffel.fr };

<_SEEN_>
	Apparently-To: ram,
	Newsgroups: mail.gue	{ DELETE };

From: root, To: root		{ BEGIN ROOT; REJECT };
<ROOT> /^Daily run output/	{ WRITE ~/var/log/york/daily.%D };
<ROOT> /^Weekly run output/	{ WRITE ~/var/log/york/weekly };
<ROOT> /^Monthly run output/	{ WRITE ~/var/log/york/monthly };

From: ram		{ BEGIN RAM; REJECT };
<RAM> To: ram		{ LEAVE };
<RAM> X-Mailer: /mailagent/	{ LEAVE };
<RAM>			{ DELETE };
.fi
.in -5
.PP
The folder directory is set to \fI~/mail\fR. All command mails are saved
in the folder \fI~/mail/cmds\fR and processed. They do not show up in
my mailbox. Mails directed to the \fIgue\fR mailing list (French Eiffel's
Users Group, namely Groupe des Utilisateurs Eiffel) are posted on the local
newsgroup \fImail.gue\fR and do not appear in my mailbox either. Any follow-up
made on this group is mailed to me by \fIinews\fR (and not directly to the
mailing list, because those mails would get back to me again and be fed to the
newsgroup, which in turn would have them mailed back to the list, and so on,
and so forth).
Hence the next rule which catches those follow-ups and bounces them to the
mailing list. Those mails will indeed come back, but the _SEEN_ rule will
simply delete them.
.PP
On my machine, the mails for \fIroot\fR are forwarded to me. However,
everyday, the \fIcron\fR daemon starts some processes to do some
administration clean-up (rotating log files, etc...), and mails the results
back. They are redirected into specific folders with the WRITE command, to
ensure they do not grow up without limit. Note the macro substitution for the
daily output (on Mondays, the output is stored in \fIdaily.1\fR for instance).
.PP
The next group of rules prevents the mail system from sending back mails when
I am in a group alias expansion. This is a \fIsendmail\fR option which I
disabled on my machine. Care is taken however to keep mails coming from the
mailagent which I receive as a blind carbon copy.
.SH CAVEAT
In order to limit the load overhead on the system, only \fBone\fR mailagent
process is allowed to run the commands. If some new mail arrives while another
mailagent is running, that mail is queued and will be processed later by the
\fImain\fR mailagent.
.PP
For the same reason, the mails sent back by the mailagent are queued by
\fIsendmail\fR, to avoid the cost of mail transfer while processing commands.
.SH FILES
.PD 0
.TP 20
~/.mailagent
configuration file for mailagent.
.TP
~/mbox.filter
mailbox used by \fIfilter\fR in case of error
.TP
~/mbox.urgent
mailbox used by \fImailagent\fR in case of error
.TP
$privlib/mailagent
directory holding templates and samples.
.TP
Log/agentlog
mailagent's log file.
.TP
Queue/agent.wait
list of mails waiting to be processed and stored outside of the mailagent's
spool directory.
.TP
Queue/qmXXXXX
mail spooled by \fIfilter\fR.
.TP
Queue/fmXXXXX
mail spooled by the \fImailagent\fR.
.TP
Hash/X/Y
hash files used by RECORD, UNIQUE, ONCE commands and vacation mode.
.PD
.SH BUGS
There is a small chance that mail arrives while the \fImain\fR mailagent
is about to finish its processing. That mail will be queued and not
processed until another mail arrives (the \fImain\fR mailagent always
processes the queue after having dealt with the message that invoked it).
.PP
A version number must currently contain a dot. Moreover, an old system
(i.e. a system with an \fIo\fR in the patches column) must have a version
number, so that the mailagent can compute the name of the directory which
holds the patches.
.PP
The lock file is deliberately ignored when \fB\-q\fR option is used (in fact,
it is ignored whenever an option is specified).
This may result in having mails processed more than once.
.PP
The mailagent is at the mercy of any \fIperl\fR bug, and there is little I
can do about it. Some spurious warnings may be emitted by the dataloaded
version, although they do not appear with the plain version.
.PP
Parsing of the rule file should be done by a real parser and not lexically.
Or at least, it should be possible to escape otherwise meaningful characters
like ';' or '}' within the rules.
.SH AUTHOR
Raphael Manfredi <ram@eiffel.com>.
.SH "SEE ALSO"
maildist($manext), mailhelp($manext), maillist($manext), mailpatch($manext),
perl(1).
!GROK!THIS!
chmod 444 mailagent.$manext
