;# $Id: actions.pl,v 2.9 92/07/14 16:49:31 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	actions.pl,v $
;# Revision 2.9  92/07/14  16:49:31  ram
;# 3.0 beta baseline.
;# 
;# 
#
# Implementation of filtering commands
#

# The "LEAVE" command
# Leave a copy of the message in the mailbox. Returns (mbox, failed_status)
sub leave {
	local($mailbox) = &mailbox_name;	# Incomming mailbox filename
	do add_log("starting LEAVE") if $loglvl > 15;
	do save($mailbox);					# Propagate return status
}

# The "SAVE" command
# Save a message in a folder. Returns (mbox, failed_status)
sub save {
	local($mailbox) = @_;			# Where mail should be saved
	local($failed) = 0;				# Printing status
	do add_log("starting SAVE $mailbox") if $loglvl > 15;
	if (open(MBOX, ">>$mailbox")) {
		do mbox_lock($mailbox);			# Lock mailbox
		# First print the Header, and add the X-Filter: line.
		(print MBOX $Header{'Head'}) || ($failed = 1);
		(print MBOX $FILTER, "\n\n") || ($failed = 1);
		(print MBOX $Header{'Body'}) || ($failed = 1);
		print MBOX "\n";				# Allow parsing by other tools
		do mbox_unlock($mailbox);		# Will close file
		# Logging only in case of error
		if ($failed) {
			do add_log("ERROR could not save mail in $mailbox") if $loglvl > 0;
			&emergency_save;
		}
	} else {
		if (-f "$mailbox") {
			do add_log("ERROR cannot append to $mailbox") if $loglvl;
		} else {
			do add_log("ERROR cannot create $mailbox") if $loglvl;
		}
		&emergency_save;
		$failed = 1;
	}
	($mailbox, $failed);			# Where save was made and failure status
}

# The "PROCESS" command
# The body of the message is expected to be in $Header{'Body'}
sub process {
	local($subj) =			$Header{'Subject'};
	local($msg_id) =		$Header{'Message-Id'};
	local($sender) =		$Header{'Reply-To'};
	local($to) =			$Header{'To'};
	local($bad) = "";		# No bad commands
	local($pack) = "auto";	# Default packing mode for sending files
	local($ncmd) = 0;		# Number of valid commands we have found
	local($dest) = "";		# Destination (where to send answers)
	local(@cmd);			# Array of all commands
	local(%packmode);		# Records pack mode for each command
	local($error) = 0;		# Error report code
	local(@body);			# Body of message

	&add_log("starting PROCESS") if $loglvl > 15;

	# If no @PATH directive was found, use $sender as a return path
	$dest = $Userpath;				# Set by an @PATH
	$dest = $sender unless $dest;
	# Remove the <> if any (e.g. path derived from Return-Path)
	$dest =~ /<(.*)>/ && ($dest = $1);

	# Debugging purposes
	&add_log("@PATH was '$Userpath' and sender was '$sender'") if $loglvl > 18;
	&add_log("computed destination: $dest") if $loglvl > 15;

	# Copy body of message in an array, one line per entry
	@body = split(/\n/, $Header{'Body'});

	# The command file contains the authorized commands
	if ($#command < 0) {			# Command file not processed yet
		open(COMMAND, "$cf'comfile") || do fatal("No command file!");
		while (<COMMAND>) {
			chop;
			$command{$_} = 1;
		}
		close(COMMAND);
	}

	line: foreach (@body) {
		# Built-in commands
		if (/^@PACK\s*(.*)/) {		# Pack mode
			$pack = $1 if $1 ne '';
			$pack = "" if ($pack =~ /[=$^&*([{}`\\|;><?]/);
		}
		s/^[ \t]@SH/@SH/;	# allow one blank only
		if (/^@SH/) {
			s/\\!/!/g;		# if uucp address, un-escape `!'
			if (/[=\$^&*([{}`\\|;><?]/) {
				s/^@SH/bad command:/;	# space after ":" will be added
				$bad .= $_;
				next line;
			}
			# Some useful substitutions
			s/@SH[ \t]*//;				# Allow leading blanks
			s/ PATH/ $dest/; 			# PATH is a macro
			s/^mial(\w*)/mail\1/;		# Common mis-spellings
			s/^mailpath/mailpatch/;
			s/^mailist/maillist/;
			# Now fetch command's name (first symbol)
			if (/^([^ \t]+)[ \t]/) {
				$first = $1;
			} else {
				$first = $_;
			}
			if (!$command{$first}) {	# if un-authorized cmd
				s/^/unknown cmd: /;		# needs a space after ":"
				$bad .= $_;
				next line;
			}
			$packmode{$_} = $pack;		# packing mode for this command
			push(@cmd, $_);				# record command
		}
	}

	# ************* Check with authoritative file ****************

	# Do not continue if an error occurred, in which case the mail will remain
	# in the queue and will be processed later on.
	return $error if $error || $dest eq '';

	# Now we are sure the mail we proceed is for us
	$sender = "<someone>" if $sender eq '';
	$ncmd = $#cmd + 1;
	if ($ncmd > 1) {
		do add_log("$ncmd commands for $sender") if ($loglvl > 11);
	} elsif ($ncmd == 1) {
		do add_log("1 command for $sender") if ($loglvl > 11);
	} else {
		do add_log("no command for $sender") if ($loglvl > 11);
	}
	foreach $fullcmd (@cmd) {
		$cmdfile = "/tmp/mess.cmd$$";
		open(CMD,">$cmdfile");
		# For our children
		print CMD "jobnum=$jobnum export jobnum\n";
		print CMD "fullcmd=\"$fullcmd\" export fullcmd\n";
		print CMD "pack=\"$packmode{$fullcmd}\" export pack\n";
		print CMD "path=\"$dest\" export path\n";
		print CMD "sender=\"$sender\" export sender\n";
		print CMD "set -x\n";
		print CMD "$fullcmd\n";
		close CMD;
		$fullcmd =~ /^[ \t]*(\w+)/;		# extract first word
		$cmdname = $1;		# this is the command name
		$trace = "$cf'tmpdir/trace.cmd$$";
		$pid = fork;						# We fork here
		if ($pid == 0) {
			open(STDOUT, ">$trace");		# Where output goes
			open(STDERR, ">&STDOUT");		# Make it follow pipe
			exec '/bin/sh', "$cmdfile";		# Don't use sh -c
		} elsif ($pid == -1) {
			# Set the error report code, and the mail will remain in queue
			# for later processing. Any @RR in the message will be re-executed
			# but it is not really important. In fact, this is going to be
			# a feature, not a bug--RAM.
			$error = 1;
			open(MAILER,"|/usr/lib/sendmail -odq -t");
			print MAILER
"To: $dest
Bcc: $cf'user
Subject: $cmdname not executed
X-Mailer: mailagent [version $mversion PL$patchlevel]

Your command was: $fullcmd

It was not executed because I could not fork. Sigh !

The command has been left in a queue and will be processed again
as soon as possible, so it is useless to resend it.

-- mailagent speaking for $cf'user";
			close MAILER;
			if ($?) {
				do add_log("ERROR cannot report failure")
					if ($loglvl > 0);
			}
			do add_log("ERROR cannot fork") if $loglvl > 0;
			return $error;		# Abort processing now--mail remains in queue
		} else {
			wait();
			if ($?) {
				open(MAILER,"|/usr/lib/sendmail -odq -t");
				print MAILER
"To: $dest
Bcc: $cf'user
Subject: $cmdname returned a non-zero status
X-Mailer: mailagent [version $mversion PL$patchlevel]

Your command was: $fullcmd
It produced the following output and failed:

";
				if (open(TRACE, "$trace")) {
					while (<TRACE>) {
						print MAILER;
					}
					close TRACE;
				} else {
					print MAILER "** SORRY - NOT AVAILABLE **\n";
					do add_log("ERROR cannot dump trace") if ($loglvl > 0);
				}
				print MAILER "\n-- mailagent speaking for $cf'user";
				close MAILER;
				if ($?) {
					do add_log("ERROR cannot report failure")
						if ($loglvl > 0);
				}
				do add_log("FAILED $fullcmd") if $loglvl > 1;
			} else {
				do add_log("OK $fullcmd") if $loglvl > 5;
			}
		}
		unlink $cmdfile, $trace;
	}

	if ($bad) {
		open(MAILER,"|/usr/lib/sendmail -odq -t");
		print MAILER
"To: $dest
Bcc: $cf'user
Subject: the following commands were not executed
X-Mailer: mailagent [version $mversion PL$patchlevel]

$bad

If $cf'name can figure out what you wanted, he may do it anyway.

-- mailagent speaking for $cf'user
";
		close MAILER;
		if ($?) {
			do add_log(
				"ERROR unable to mail back bad commands from $sender"
			) if ($loglvl > 0);
		}
		do add_log("bad commands from $sender") if ($loglvl > 5);
	}

	do add_log("all done for $sender") if ($loglvl > 11);
	$error;		# Return error report (0 for ok)
}

# The "MESSAGE" command
sub message {
	local($msg) = @_;			# Vacation message to be sent back
	local(@head) = (
		"To: %r (%N)",
		"Subject: Re: %R"
	);
	do send_message($msg, *head);
}

# The "NOTIFY" command
sub notify {
	local($msg, $address) = @_;
	# Protect all '%' in the address (subject to macro substitution)
	$address =~ s/%/%%/g;
	local(@head) = (
		"To: $address",
		"Subject: %s (notification)"
	);
	do send_message($msg, *head);
}

# Send a given message to somebody, as specified in the given header
# The message and the header are subject to macro substitution
sub send_message {
	local($msg, *header) = @_;	# Message to send, header of message
	unless (-f "$msg") {
		do add_log("cannot find message $msg") if $loglvl > 0;
		return 1;
	}
	unless (open(MSG, "$msg")) {
		do add_log("cannot open message $msg") if $loglvl > 0;
		return 1;
	}
	unless (open(MAILER,"|/usr/lib/sendmail -odq -t")) {
		do add_log("cannot run sendmail to send message") if $loglvl > 0;
		return 1;
	}

	# Construction of value for the %T macro
	local($macro_T);			# Default value of macro %T is overwritten
	local($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime, $mtime,
		$ctime,$blksize,$blocks) = stat($msg);
	local($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
			localtime($mtime);
	local($this_year) = (localtime(time))[5];
	# Do not put the year in %T if it is the same as the current one.
	++$mon;						# Month in the range 1-12
	if ($this_year != $year) {
		$macro_T = sprintf("%.2d/%.2d/%.2d", $year, $mon, $mday);
	} else {
		$macro_T = sprintf("%.2d/%.2d", $mon, $mday);
	}

	# Header construction. If the file contains a header at the top, it is
	# added to the one we already have by default. Identical fields are
	# overwritten with the one found in the file.
	if (&header_found($msg)) {	# Top of message is a header
		local(@newhead);		# New header is constructed here
		local($field);
		while (<MSG>) {			# Read the header then
			last if /^$/;		# End of header
			chop;
			push(@newhead, $_);
			if (/^([\w\-]+):/) {
				$field = $1;
				@head = grep(!/^$field:/, @head);	# Field is overwritten
			}
		}
		foreach (@newhead) {
			push(@head, $_);
		}
	}
	push(@head, $FILTER);		# Avoid loops: replying to ourselves or whatever
	foreach $line (@head) {
		do macros_subst(*line);	# In-place macro substitutions
		print MAILER "$line\n";	# Write header
	}
	print MAILER "\n";			# Header separated from body
	# Now write the body
	local($tmp);				# Because of a bug in perl 4.0 PL19
	while ($tmp = <MSG>) {
		next if /^$/ && $. == 1;	# Escape sequence to protect header
		do macros_subst(*tmp);		# In-place macro substitutions
		print MAILER $tmp;			# Write message line
	}

	# Close pipe and check status
	close MSG;
	close MAILER;
	local($status) = $?;
	unless ($status) {
		if ($loglvl > 2) {
			local($dest) = $head[0];	# The To: header line
			($dest) = $dest =~ m|^To:\s+(.*)|;
			do add_log("SENT message to $dest");
		}
	} else {
		do add_log("ERROR could not mail back $msg") if $loglvl > 1;
	}
	$status;		# 0 for success
}

# The "FORWARD" command
sub forward {
	local($addresses) = @_;			# Address(es) mail should be forwarded to
	local($address) = &email_addr;	# Address of user
	# Any address included withing "" is in fact a file name where actual
	# forwarding addresses are found.
	$addresses = &complete_addr($addresses);	# Process "include-requests"
	unless (open(MAILER,"|/usr/lib/sendmail -odq $addresses")) {
		do add_log("cannot run sendmail to forward message") if $loglvl > 0;
		return 1;
	}
	local(@addr) = split(' ', $addresses);
	print MAILER "Resent-From: $address\n";
	print MAILER "Resent-To: ", join(', ', @addr), "\n";
	# Protect Sender: and Resent-: lines in the original message
	foreach (split(/\n/, $Header{'Head'})) {
		next if /^From\s+(\S+)/;
		s/^Sender:\s*(.*)/Original-Sender: $1/;
		s/^Resent-([\w\-]+):\s*(.*)/Original-Resent-$1: $2/;
		print MAILER $_, "\n";
	}
	print MAILER $FILTER, "\n";
	print MAILER "\n";
	print MAILER $Header{'Body'};
	close MAILER;
	local($failed) = $?;		# Status of forwarding
	if ($failed) {
		do add_log("ERROR could not forward to $addresses") if $loglvl > 1;
	}
	$failed;		# 0 for success
}

# The "BOUNCE" command
sub bounce {
	local($addresses) = @_;			# Address(es) mail should be bounced to
	# Any address included withing "" is in fact a file name where actual
	# bouncing addresses are found.
	$addresses = &complete_addr($addresses);	# Process "include-requests"
	unless (open(MAILER,"|/usr/lib/sendmail -odq $addresses")) {
		do add_log("cannot run sendmail to bounce message") if $loglvl > 0;
		return 1;
	}
	# Protect Sender: lines in the original message
	foreach (split(/\n/, $Header{'Head'})) {
		next if /^From\s+(\S+)/;
		s/^Sender:\s*(.*)/Original-Sender: $1/;
		print MAILER $_, "\n";
	}
	print MAILER $FILTER, "\n";
	print MAILER "\n";
	print MAILER $Header{'Body'};
	close MAILER;
	local($failed) = $?;		# Status of forwarding
	if ($failed) {
		do add_log("ERROR could not bounce to $addresses") if $loglvl > 1;
	}
	$failed;		# 0 for success
}

# The "POST" command
sub post {
	# Option parsing: a -l restricts distribution to local
	local($localdist) = 0;
	$localdist = 1 if ($_[0] =~ s/^\s*-l\s+//);
	local($newsgroups) = @_;		# Newsgroup(s) mail should be posted to
	local($address) = &email_addr;	# Address of user
	unless (open(NEWS,"| $inews -h")) {
		do add_log("cannot run $inews to post message") if $loglvl > 0;
		return 1;
	}
	do add_log("distribution of posting is local")
		if $loglvl > 18 && $localdist;
	# Protect Sender: lines in the original message and clean-up header
	local($last_was_header);		# Set to true when header is skipped
	foreach (split(/\n/, $Header{'Head'})) {
		s/^Sender:\s*(.*)/Original-Sender: $1/;
		next if /^From\s/;					# First From line...
		if (
			/^To:/ ||
			/^Cc:/ ||
			/^Apparently-To:/ ||
			/^Distribution:/ ||				# No mix-up, please
			/^X-Mailer:/ ||					# Mailer identification
			/^Newsgroups:/ ||				# Reply from news reader
			/^Return-Receipt-To:/ ||		# Sendmail's acknowledgment
			/^Received:/ ||					# We want to remove received
			/^Errors-To:/ ||				# Error report redirection
			/^Resent-[\w-]*:/				# Resent tags
		) {
			$last_was_header = 1;			# Mark we discarded the line
			next;							# Line is skipped
		}
		next if /^\s/ && $last_was_header;	# Skip removed header continuations
		$last_was_header = 0;				# We decided to keep header line
		print NEWS $_, "\n";
	}
	# If no subject is present, fake one to make inews happy
	unless (defined($Header{'Subject'}) && $Header{'Subject'} ne '') {
		do add_log("WARNING no subject, faking one") if $loglvl > 5;
		print NEWS "Subject: <none>\n";
	}
	print NEWS "Newsgroups: $newsgroups\n";
	print NEWS "Distribution: local\n" if $localdist;
	# Avoid loops: inews may forward to sendmail
	print NEWS $FILTER, "\n";
	print NEWS "\n";
	print NEWS $Header{'Body'};
	close NEWS;
	local($failed) = $?;		# Status of forwarding
	if ($failed) {
		do add_log("ERROR could not post to $newsgroups") if $loglvl > 1;
	}
	$failed;		# 0 for success
}

# The "SPLIT" command
# This routine is RFC-934 compliant and will correctly burst digests produced
# with this RFC in mind. For instance, MH produces RFC-934 style digest.
# However, in order to reliably split non RFC-934 digest, some extra work is
# performed to ensure a meaningful output.
sub split {
	# Option parsing: a -i splits "inplace", i.e. acts as a saving if the split
	# is fully successful. A -d discards the leading part. A -q queues messsages
	# instead of filling them into a folder.
	$_[0] =~ s/^\s*-([deiw]+)//;		# Remove options
	local($opt) = $1;
	local($inplace) = $opt =~ /i/;	# Inplace (original marked saved)
	local($discard) = $opt =~ /d/;	# Discard digest leading part
	local($empty) = $opt =~ /e/;	# Discard leading digest only if empty
	local($watch) = $opt =~ /w/;	# Watch digest closely
	$_[0] =~ s/^\s+//;				# Trim leading spaces
	local($folder) = $_[0];		# Folder to save messages
	local(@leading);			# Leading part of the digest
	local(@header);				# Looked ahead header
	local($found_header) = 0;	# True when header digest was found
	local($look_header) = 0;	# True when we are looking for a mail header
	local($found_end) = 0;		# True when end of digest found
	local($valid);				# Return value from header checking package
	local($failed) = 0;			# Queuing status for each mail item
	local(@body);				# Body of extracted mail
	local($item) = 0;			# Count digest items found
	local($not_rfc934) = 0;		# Is digest RFC-934 compliant?
	local($_);
	# Start digest parsing. According to RFC-934, we could only look for a
	# single '-' as encapsulation boundary, but for safety we look for at least
	# three consecutive ones.
	foreach (split(/\n/, $Header{'All'})) {
		push(@leading, $_) unless $found_header;
		push(@body, $_) if $found_header;
		if (/^---/) {			# Start looking for mail header
			$look_header = 1;	# Focus on mail headers now
			# We are withing the body of a digest and we've just reached
			# what may be the end of a message, or the end of the leading part.
			@header = ();		# Reset look ahead buffer
			&header'reset;		# Reset header checking package
			next;
		}
		next unless $look_header;
		# Record lines we find, but skip possible blank lines after dash.
		# Note that RFC-934 does not make spaces compulsory after each
		# encapsulation boundary (EB) but they are allowed nonetheless.
		next if /^\s*$/ && 0 == @header;
		$found_end = 0;			# Maybe it's not garbage after all...
		$valid = &header'valid($_);
		if ($valid == 0) {		# Not a valid header
			$look_header = 0;	# False alert
			$found_end = 1;		# Garbage after last EB is to be ignored
			if ($watch) {
				# Strict RFC-934: if an EB is followed by something which does
				# not prove to be a valid header but looked like one, enough
				# to have some lines collected into @header, then signal it.
				++$not_rfc934 unless 0 == @header;
			} else {
				# Don't be too scrict. If what we have found so far *may be* a
				# header, then yes, it's not RFC-934. Otherwise let it go.
				++$not_rfc934 if $header'maybe;
			}
			next;
		} elsif ($valid == 1) {	# Still in header
			push(@header, $_);	# Record header lines
			next;
		}
		# Coming here means we reached the end of a valid header
		push(@header, '');		# Blank header line
		if (!$found_header) {
			if ($empty) {
				$failed |= &save_mail(*leading, $folder)
					unless &empty_body(*leading) || $discard;
			} else {
				$failed |= &save_mail(*leading, $folder) unless $discard;
			}
			undef @leading;		# Not needed any longer
			$item++;			# So that 'save_mail' starts logging items
		}
		# If there was already a mail being collected, save it now, because
		# we are sure it is followed by a valid mail.
		$failed |= &save_mail(*body, $folder) if $found_header;
		$found_header = 1;		# End of header -> this is truly a digest
		$look_header = 0;		# We found our header
		&header'clean(*header);	# Ensure minimal set of header
		@body = @header;		# Copy headers in mail body for next message
	}

	return -1 unless $found_header;	# Message was not in digest format

	# Save last message, making sure to add a final dash line if digest did
	# not have one: There was one if $look_header is true. There was also
	# one if $found_end is true.
	push(@body, '---') unless $look_header || $found_end;

	# If the -w option was used, we look closely at the supposed trailing
	# garbage. If the length is greater than 100 characters, then maybe we
	# are missing something here...
	if ($watch) {
		local($idx) = $#body;
		$_ = $body[$idx];			# Get last line
		@header = ();				# Reset "garbage collector"
		unless (/^---/) {			# Do not go on if end of digest truly found
			for (; $idx >= 0; $idx--) {
				$_ = $body[$idx];
				last if /^---/;		# Reached end of presumed trailing garbage
				unshift(@header, $_);
			}
		}
	}

	# Now save last message
	$failed |= &save_mail(*body, $folder);

	# If we collected something into @header and if it is big enough, save it
	# as a trailing message.
	if ($watch && length(join('', @header)) > 100) {
		&add_log("NOTICE [$mfile] has trailing garbage...") if $loglvl > 6;
		@body = @header;			# Copy saved garbage
		@header = ();				# Now build final garbage headers
		$header[0] = 'Subject: ' . $Header{'Subject'} . ' (trailing garbage)';
		&header'clean(*header);		# Build other headers
		unshift(@body, '') unless $body[0] =~ s/^\s*$//;	# Ensure EOH
		foreach (@body) {
			push(@header, $_);
		}
		push(@header, '---');
		$failed |= &save_mail(*header, $folder);
	}

	$failed + 0x2 * $inplace + 0x4 * ($folder =~ /^\s*$/)
		+ 0x8 * ($not_rfc934 > 0);
}

# The "RUN" command and its friends
# Start a shell command and mail any output back to the user.
sub shell_command {
	local($program, $input, $feedback) = @_;
	$program =~ s/^\s*~/$cf'home/;	# ~ substitution
	$program =~ s/\b~/$cf'home/g;	# ~ substitution as first letter in word
	$SIG{'PIPE'} = 'popen_failed';	# Protect against naughty program
	$SIG{'ALRM'} = 'alarm_clock';	# Protect against loops
	alarm 3600;						# At most one hour of processing
	eval "&execute_command(q\0$program\0, $input, $feedback)";
	alarm 0;						# Disable alarm timeout
	$SIG{'PIPE'} = 'emergency';		# Restore initial value
	$SIG{'ALRM'} = 'DEFAULT';		# Restore default behaviour
	if ($@ =~ /^failed/) {			# Something went wrong?
		do add_log("ERROR couldn't run '$program'") if $loglvl > 0;
		return 1;					# Failed
	} elsif ($@ =~ /^aborted/) {	# Writing to program failed
		do add_log("WARNING pipe closed by '$program'") if $loglvl > 5;
		return 1;					# Failed
	} elsif ($@ =~ /^feedback/) {	# Feedback failed
		do add_log("WARNING no feedback occurred") if $loglvl > 5;
		return 1;					# Failed
	} elsif ($@ =~ /^alarm/) {		# Timeout
		do add_log("WARNING time out received") if $loglvl > 5;
		return 1;					# Failed
	} elsif ($@) {
		do add_log("ERROR $@") if $loglvl > 0;
		return 1;					# Failed
	}
	0;			# Everything went fine
}

# Abort execution of command when popen() fails or program dies abruptly
sub popen_failed {
	unlink "$trace" if -f "$trace";
	die "$error\n";
}

# When an alarm call is received, we should be in the 'execute_command'
# routine. The $pid variable holds the pid number of the process to be killed.
sub alarm_clock {
	if ($trace ne '' && -f "$trace") {		# We come from execute_command
		local($status) = "terminated";		# Process was terminated
		if (kill "SIGTERM", $pid) {			# We could signal our child
			sleep 30;						# Give child time to die
			unless (kill "SIGTERM", $pid) {	# Child did not die yet ?
				unless (kill "SIGKILL", $pid) {
					do add_log("ERROR could not kill process $pid: $!")
						if $loglvl > 1;
				} else {
					$status = "killed";
					do add_log("KILLED process $pid") if $loglvl > 4;
				}
			} else {
				do add_log("TERMINATED process $pid") if $loglvl > 4;
			}
		} else {
			$status = "unknwon";			# Process died ?
			do add_log("ERROR coud not signal process $pid: $!")
				if $loglvl > 1;
		}
		do mail_back();				# Mail back any output we have so far
		unlink "$trace";			# Remove output of command
	}
	die "alarm call\n";				# Longjmp to shell_command
}

# Execute the command, ran in an eval to protect against SIGPIPE signals
sub execute_command {
	local($program, $input, $feedback) = @_;
	local($trace) = "$cf'tmpdir/trace.run$$";	# Where output goes
	local($error) = "failed";				# Error reported by popen_failed
	pipe(READ, WRITE);						# Open a pipe
	local($ppid) = $$;						# Pid of parent process
	local($pid) = fork;						# We fork here
	if ($pid == 0) {						# Child process
		alarm 0;
		close WRITE;						# The child reads from pipe
		open(STDIN, "<&READ");				# Redirect stdin to pipe
		close READ if $input == $NO_INPUT;	# Close stdin if needed
		unless (open(STDOUT, ">$trace")) {	# Where output goes
			do add_log("WARNING couldn't create $trace") if $loglvl > 5;
			if ($feedback == $FEEDBACK) {	# Need trace if feedback
				kill 'SIGPIPE', $ppid;		# Parent still waiting
				exit 1;
			}
		}
		open(STDERR, ">&STDOUT");			# Make it follow pipe
		exec "$program";					# Run the program now
		do add_log("ERROR couldn't exec '$program': $!") if $loglvl > 1;
		kill 'SIGPIPE', $ppid;				# Parent still waiting
		exit 1;
	} elsif ($pid == -1) {
		do add_log("ERROR couldn't fork") if $loglvl > 0;
		return;
	}
	close READ;								# The parent writes to its child
	# In case 'sleep' is inplemented using an alarm call, take precautions...
	local($remaining) = alarm 0;			# Stop alarm, save remaining time
	sleep 5;								# Let the child initialize
	alarm $remaining;						# Restore alarm clock
	$error = "aborted";						# Error reported by popen_failed
	select(WRITE);
	$| = 1;									# Hot pipe wanted
	select(STDOUT);
	# Now feed the program with the mail
	if ($input == $BODY_INPUT) {			# Pipes body
		print WRITE $Header{'Body'};
	} elsif ($input == $MAIL_INPUT) {		# Pipes the whole mail
		print WRITE $Header{'All'};
	} elsif ($input == $HEADER_INPUT) {		# Pipes the header
		print WRITE $Header{'Head'};
	}
	close WRITE;							# Close input, before waiting!
	wait();									# Wait for our child
	local($status) = $? ? "failed" : "ok";
	if ($?) {
		# Log execution failure and return to shell_command via die if some
		# feedback was to be done.
		do add_log("ERROR execution failed for '$program'") if $loglvl > 1;
		if ($feedback == $FEEDBACK) {		# We wanted feedback
			do mail_back();					# Mail back any output
			unlink "$trace";				# Remove output of command
			die "feedback\n";				# Longjmp to shell_command
		}
	}
	&handle_output;			# Take appropriate action with command output
	unlink "$trace";		# Remove output of command
}

# If no feedback is wanted, simply mail the output of the commands to the
# user. However, in case of feedback, we have to update the values of
# %Header in the entries 'All', 'Body' and 'Head'. Note that the other
# header fields are left untouched. Only a RESYNC can synchronize them
# (this makes sense only for a FEED command, of course).
# Uses $feedback from execute_command
sub handle_output {
	if ($feedback == $NO_FEEDBACK) {
		&mail_back;						# Mail back any output
	} elsif ($feedback == $FEEDBACK) {
		&feed_back;						# Feed result back into %Header
	}
}

# Mail back the contents of the trace file (output of program), if not empty.
# Uses some local variables from execute_command
sub mail_back {
	local($size) = -s "$trace";				# Size of output
	return unless $size;					# Nothing to be done if no output
	local($std_input);						# Standard input used
	$std_input = "none" if $input == $NO_INPUT;
	$std_input = "mail body" if $input == $BODY_INPUT;
	$std_input = "whole mail" if $input == $MAIL_INPUT;
	$std_input = "header" if $input == $HEADER_INPUT;
	local($program_name) = $program =~ m|^(\S+)|;
	open(MAILER,"|/usr/lib/sendmail -odq -t");
	print MAILER
"To: $cf'user
Subject: Output of your '$program_name' command ($status)
X-Mailer: mailagent [version $mversion PL$patchlevel]

Your command was: $program
Input: $std_input
Status: $status

It produced the following output:

";
	unless (open(TRACE, "$trace")) {
		do add_log("ERROR couldn't reopen $trace") if $loglvl > 1;
		print MAILER "*** SORRY -- NOT AVAILABLE ***\n";
	} else {
		while (<TRACE>) {
			print MAILER;
		}
		close TRACE;
	}
	close MAILER;
	unless ($?) {
		do add_log("SENT output of '$program_name' to $cf'user ($size bytes)")
			if $loglvl > 2;
	} else {
		do add_log("ERROR couldn't send $size bytes to $cf'user") if $loglvl;
	}
}

# Feed back output of a command in the %Header data structure.
# Uses some local variables from execute_command
sub feed_back {
	unless (open(TRACE, "$trace")) {
		do add_log("ERROR couldn't feed back from $trace: $!") if $loglvl > 1;
		unlink "$trace";				# Maybe I should leave it around
		die "feedback\n";				# Return to shell_command
	}
	local($temp) = ' ' x 2000;			# Temporary storage (pre-extended)
	$temp = '';
	if ($input == $BODY_INPUT) {		# We have to feed back the body only
		while (<TRACE>) {
			s/^From\s/>From$1/;			# Protect potentially dangerous lines
			$temp .= $_;
		}
	} else {
		local($head) = ' ' x 500;		# Pre-extend header
		$head = '';
		while (<TRACE>) {
			if (1../^$/) {
				$head .= $_ unless /^$/;
			} else {
				s/^From\s/>From$1/;		# Protect potentially dangerous lines
				$temp .= $_;
			}
		}
		$Header{'Head'} = $head;
	}
	close TRACE;
	$Header{'Body'} = $temp unless $input == $HEADER_INPUT;
	$Header{'All'} = $Header{'Head'} . "\n" . $Header{'Body'};
}

# Feed output back into $Back variable (used by BACK command). Typically, the
# BACK command is used with RUN, though any other command is allowed (but does
# not always make sense).
# NB: This routine:
#  - Is never called explicitely but via a type glob through *handle_output
#  - Uses some local variables from execute_command
sub xeq_back {
	unless (open(TRACE, "$trace")) {
		do add_log("ERROR couldn't feed back from $trace: $!") if $loglvl > 1;
		unlink "$trace";				# Maybe I should leave it around
		die "feedback\n";				# Return to shell_command
	}
	while (<TRACE>) {
		chop;
		next if /^\s*$/;
		$Back .= $_ . '; ';				# Replace \n by ';' separator
	}
	close TRACE;
}

# The "RESYNC" command
# Resynchronizes the %Header entries by reparsing the 'All' entry
sub header_resync {
	# Clean up all the non-special entries
	foreach $key (keys %Header) {
		next if $Pseudokey{$key};		# Skip pseudo-header entries
		delete $Header{$key};
	}
	# There is some code duplication with parse_mail()
	local($lines) = 0;
	local($first_from);
	local($last_header);
	local($in_header) = 1;					# Bug in the range operator
	foreach (split(/\n/, $Header{'All'})) {
		if ($in_header) {					# Still in header of message
			$in_header = 0 if /^$/;			# End of header
			if (/^\s/) {					# It is a continuation line
				s/^\s+/ /;					# Swallow multiple spaces
				$Header{$last_header} .= "\n$_" if $last_header ne '';
			} elsif (/^([\w-]+):\s*(.*)/) {	# We found a new header
				$last_header = $1;
				# Multiple headers like 'Received' are separated by a new-
				# line character. All headers end on a non new-line.
				if ($Header{$1} ne '') {
					$Header{$1} .= "\n$2";
				} else {
					$Header{$1} .= $2;
				}
			} elsif (/^From\s+(\S+)/) {		# The very first From line
				$first_from = $1;
			}
		} else {
			$lines++;						# One more line in body
		}
	}
	do header_check($first_from, $lines);	# Sanity checks
}

# The "STRIP" and "KEEP" commands
# Removes or keeps some headers and update the Header structure
sub alter_header {
	local($headers, $action) = @_;
	local(@list) = split(/\s/, $headers);
	local(@head) = split(/\n/, $Header{'Head'});
	local(@newhead);				# The constructed header
	local($last_was_altered) = 0;	# Set to true when header is altered
	local($matched);				# Did any header matched ?
	foreach (@head) {
		if (/^From\s/) {			# First From line...
			push(@newhead, $_);		# Keep it anyway
			next;
		}
		unless (/^\s/) {			# If not a continuation line
			$last_was_altered = 0;	# Reset header alteration flag
			$matched = 0;			# Assume no match
			foreach $h (@list) {	# Loop over to-be-altered lines
				$h =~ s/:$//;		# Remove trailing ':' if any
				if (/^$h:/) {		# We found a line to be removed/kept
					$matched = 1;
					last;
				}
			}
			$last_was_altered = $matched;
			next if $matched && $action == $HD_SKIP;
			next if !$matched && $action == $HD_KEEP;
		}
		if ($action == $HD_SKIP) {
			next if /^\s/ && $last_was_altered;		# Skip header continuations
		} else {									# Action is $HD_KEEP
			next if /^\s/ && !$last_was_altered;	# Header was not kept
		}
		push(@newhead, $_);						# Add line to the new header
	}
	$Header{'Head'} = join("\n", @newhead) . "\n";
}

# The "ANNOTATE" command
sub annotate_header {
	local($field, $value, $date) = @_;	# Field, value and date flag.
	if ($value eq '' && $date ne '') {	# No date and no value for field!
		&add_log("WARNING no value for '$field' annotation") if $loglvl > 5;
		return 1;
	}
	if ($field eq '') {				# No field specified!
		&add_log("WARNING no field specified for annotation") if $loglvl > 5;
		return 1;
	}
	local($annotation) = '';		# Annotation made
	$annotation = "$field: " . &header'fake_date . "\n" unless $date;
	$annotation .= "$field: $value\n" if $value;
	$Header{'Head'} .= $annotation;
	0;
}

# The "TR" and "SUBST" commands
sub alter_value {
	local($variable, $op) = @_;	# Variable and operation to performed
	local($lvalue);				# Perl variable to be modified
	# We may modify a variable or a backreference (not read-only as in perl)
	if ($variable =~ s/^#//) {
		$lvalue = '$Variable{\''.$variable.'\'}';
	} elsif ($variable =~ /^\d\d?$/) {
		$variable = int($variable) - 1;
		$lvalue = '$Backref[' . $variable . ']';
	} else {
		&add_log("ERROR incorrect variable name '$variable'") if $loglvl > 1;
		return 1;
	}
	&add_log("running $lvalue =~ $op") if $loglvl > 19;
	eval $lvalue . " =~ $op";
	&add_log("ERROR operation $op failed: $@") if $@ && $loglvl > 1;
	0;
}

# Given a list of addresses separated by white spaces, return a new list of
# addresses, but with "include-request" processed.
sub complete_addr {
	local(@addr) = split(' ', $_[0]);	# Original list
	local(@result);						# Where result list is built
	local($filename);					# Name of include file
	local($_);
	foreach $addr (@addr) {
		if ($addr !~ /^"/) {			# Address not enclosed within ""
			push(@result, $addr);		# Kept as-is
		} else {
			($filename) = $addr =~ /^"(.*)"$/;
			$filename = &locate_file($filename);
			if ($filename && open(ADDRESSES, "$filename")) {
				while (<ADDRESSES>) {
					next if /^\s*#/;	# Skip shell comments
					chop;
					s/^\s+//;			# Remove leading spaces
					push(@result, $_);
				}
				close ADDRESSES;
			} elsif ($filename) {		# Could not open file
				&add_log("WARNING couldn't open $filename for addresses: $!")
					if $loglvl > 4;
			} else {
				&add_log("WARNING incorrect file inclusion request")
					if $loglvl > 4;
			}
		}
	}
	join(' ', @result);		# Return space separated addresses
}

# Save digest mail into a folder, or queue it if no folder is provided
# Uses the variable '$item' from 'split' to log items.
sub save_mail {
	local(*array, $folder) = @_;	# Where mail is and where to put it
	local($length) = 0;				# Length of the digest item
	local($mbox, $failed, $log_message);
	local($_);
	# Go back to the previous dash line, removing it from the body part
	# (it's only a separator). In the process, we also remove any looked ahead
	# header which belongs to the next digest item.
	do {
		$_ = pop(@array);			# Remove what belongs to next digest item
	} while !/^---/;
	# It is recommended in RFC-934 that all leading EB be escaped by a leading
	# '- ' sequence, to allow nested forwarding. However, since the message
	# we are dealing with might not be RFC-934 compliant, we are only removing
	# the leading '- ' if it is followed by a '-'. We also use the loop to
	# escape all potentially dangerous From lines.
	local($last_was_space);
	foreach (@array) {
		s/^From\s+(\S+)/>From $1/ if $last_was_space;
		s/^- -/-/;					# This is the EB escape in RFC-934
		$last_was_space = /^$/;		# From is dangerous after blank line
	}
	# Now @array holds the whole digest item
	if ($folder =~ /^\s*$/) {		# No folder means we have to queue message
		$failed = &qmail(*array);
		$log_message = 'mailagent\'s queue';
		foreach (@array) {
			$length += length($_) + 1;	# No trailing new-lines
		}
	} else {
		# Looks like we have to save the message in a folder. I cannot really
		# ask for a local variable named %Header because emergency routines
		# use it to save mail (they expect the whole mail in $Header{'All'}).
		# However, if something goes wrong, we'll get back to the filter main
		# loop and a LEAVE (default action) will be executed, taking the
		# current values from 'Head' and 'Body'. Hence the following:

		local(%NHeader);
		$NHeader{'All'} = $Header{'All'};
		local(*Header) = *NHeader;	# From now on, we really work on %NHeader
		local($in_header) = 1;		# True while in message header
		local($first_from);			# First From line

		# Fill in %Header strcuture, which is expected by save(): header in
		# entry 'Head' and body in entry 'Body'.
		foreach (@array) {
			if ($in_header) {
				$in_header = 0 if /^$/;
				next if /^$/;
				$Header{'Head'} .= $_ . "\n";
				$first_from = $_ if /^From\s+\S+/;
				next;
			}
			$Header{'Body'} .= $_ . "\n";
		}
		$Header{'Head'} = "$FAKE_FROM\n" .  $Header{'Head'} unless $first_from;

		# Now save into folder
		($mbox, $failed, $log_message) = &run_saving($folder, $FOLDER_APPEND);

		# Keep track in the logfile of the length of the digest item.
		$length = length($Header{'Head'}) + length($Header{'Body'}) + 1;
	}
	if ($failed) {
		if ($loglvl > 2) {
			local($s) = $length == 1 ? '' : 's';
			&add_log("ERROR unable to save #$item ($length byte$s)") if $item;
			&add_log("ERROR unable to save preamble ($length byte$s)")
				unless $item;
		}
	} else {
		if ($loglvl > 7) {
			local($s) = $length == 1 ? '' : 's';
			&add_log("SPLIT #$item in $log_message ($length byte$s)") if $item;
			&add_log("SPLIT preamble in $log_message ($length byte$s)")
				unless $item;
		}
	}
	++$item if $item;		# Count items, but not preamble (done by 'split')
	$failed;				# Propagate failure status
}

# Check body message (typically head of digest message) and return 1 if its
# body is empty, 0 otherwise.
sub empty_body {
	local(*ary) = @_;
	local(@array) = @ary;		# Work on a copy
	local($_);
	local($is_empty) = 1;
	do {
		$_ = pop(@array);		# Remove what belongs to next digest item
	} while !/^---/;
	do {
		$_ = shift(@array);		# Remove the whole header
	} while !/^$/;
	foreach (@array) {
		$is_empty = 0 unless /^\s*$/;
		last unless $is_empty;
	}
	$is_empty;
}

