;# $Id: analyze.pl,v 2.9 92/07/14 16:49:36 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	analyze.pl,v $
;# Revision 2.9  92/07/14  16:49:36  ram
;# 3.0 beta baseline.
;# 
;# 
#
# Analyzing mail
#

# Special users. Note that as login name matches are done in a case-insensitive
# manner, there is no need to upper-case any of the followings.
sub init_special {
	%Special = (
		'root', 1,				# Super-user
		'uucp', 1,				# Unix to Unix copy
		'daemon', 1,			# Not a real user, hopefully
		'news', 1,				# News daemon
		'postmaster', 1,		# X-400 mailer-daemon name
		'newsmaster', 1,		# My convention for news administrator--RAM
		'usenet', 1,			# Aka newsmaster
		'mailer-daemon', 1,		# Sendmail
		'nobody', 1				# Nobody we've heard of
	);
}

# Initialize global variables, before analyzing each mail
sub init_global_state {
	undef %Variable;			# User-defined variables
}

# This is the heart of the mail agent
# Scan the message in $file_name and apply the filtering rules
sub analyze_mail {
	local($file) = shift(@_);	# Mail file to be parsed
	local($mode);				# Mode (optional)
	local($wmode)= "INITIAL";	# Working mode (the mode we are in)
	local($selector);			# Selector (mandatory)
	local($rulentry);			# Entry in rule H table
	local($pattern);			# Pattern for selection, as written in rules
	local($action);				# Related action
	local($last_selector);		# Last used selector
	local($rules);				# A copy of the rules
	local($matched);			# Flag set to true if a rule is matched
	local(%Matched);			# Records the selectors which have been matched
	local($header);				# Header entry name to look for in Header table
	local($status);				# Status returned by xeqte
	local($ever_matched) = 0;	# Did we ever matched a single saving rule ?
	local($ever_saved) = 0;		# Did we ever saved a message ?
	local($ever_seen) = 0;		# Did we ever enter seen mode ?
	local($vacation) = 1;		# Vacation message allowed a priori
	local(@Executed);			# Records already executed rules

	# First parse the mail and fill in the %Header table
	&init_global_state;			# Initializes global variables
	&parse_mail($file);			# Parse the mail and fill-in H tables
	return 0 unless defined $Header{'All'};
	&reception if $loglvl > 8;	# Log mail reception
	&run_builtins;				# Execute builtins, if any were found

	# Now analyze the mail. If there is already a X-Filter header, then the
	# mail has already been processed. In that case, the default action is
	# performed: leave it in the incomming mailbox with no further action.
	# This should prevent nasty loops.
	do add_log ("analyzing mail") if $loglvl > 18;
	$header = $Header{'X-Filter'};				# Mulitple occurences possible
	if ($header ne '') {						# Hmm... already filtered...
		local(@filter) = split(/\n/, $header);	# Look for each X-Filter
		local($address) = &email_addr;			# Our e-mail address
		local($done) = 0;						# Already processed ?
		foreach (@filter) {						# Maybe we'll find ourselves
			if (/mailagent.*for (\S+)/) {		# Mark left by us ?
				$done = 1 if $1 eq $address;	# Yes, we did that
				$* = 1;
				# Remove that X-Filter line, LEAVE will add one anyway
				$Header{'Head'} =~ s/^X-Filter:\s*mailagent.*for $address\n//;
				$* = 0;
				last;
			}
		}
		if ($done) {			# We already processed that message
			do add_log("NOTICE already filtered, entering seen mode")
				if $loglvl > 5;
			$wmode = '_SEEN_';	# This is a special mode
			$ever_seen = 1;		# This will prevent vacation messages
			&s_seen;			# Update statistics
		}
	}

	# The @Executed array records whether a specified action for a rule was
	# executed. Loops are possible via the RESTART action, and as there is
	# almost no way to exit from such a loop (there is one with FEED and RESYNC)
	# I decided to prohibit them. Hence a given action is allowed to be executed
	# only once during a mail analysis (modulo each possible working mode).
	# For a rule number n, $Executed[n] is a collection of modes in which the
	# rule was executed, comma separated.
	$Executed[$#Rules] = '';					# Pre-extend array

	# Order wrt the one in the rule file is guaranteed. I use a for construct
	# with indexed access to be able to restart from the beginning upon
	# execution of RESTART. This also helps filling in the @Executed array.
	local($i, $j);
	rule: for ($i = 0; $i <= $#Rules; $i++) {
		$j = $i + 1;
		$_ = $Rules[$i];
		# The %Matched array records the boolean value associated with each
		# possible selector. If two identical selector are found, the values
		# are OR'ed (and we stop evaluating as soon as one is true). Otherwise,
		# the values are AND'ed (for different selectors, but all are evaluated
		# in case we later find another identical selectors -- no sort is done).
		undef %Matched;							# Reset matching patterns
		$rules = $_;							# Work on a copy
		$rules =~ s/^(.*){// && ($mode = $1);		# First word is the mode
		$rules =~ s/\s*(.*)}// && ($action = $1);	# Followed by action
		$mode =~ s/\s*$//;							# Remove trailing spaces
		$rules =~ s/^\s+//;						# Remove leading spaces
		$last_selector = "";					# Last selector used

		# Make sure we are in the correct mode. The $mode variable holds a
		# list of comma-separated modes. If the working mode is found in it
		# then the rules apply. Otherwise, skip them.
		do add_log ("in mode '$wmode' for $mode") if $loglvl > 19;
		$mode = "," . $mode . ",";
		# The special ALL mode matches anything but the other sepcial mode
		# for already filtered messages.
		unless ($mode =~ /,ALL,/) {
			next rule unless $mode =~ /,$wmode,/;
		} else {
			next rule if $wmode eq '_SEEN_' && $mode !~ /,_SEEN_,/;
		}

		# Now loop over all the keys and apply the patterns in turn
		&reset_backref;						# Reset backreferences
		foreach $key (split(/ /, $rules)) {
			$rulentry = $Rule{$key};
			$rulentry =~ s/^\s*([^\/]*:)// && ($selector = $1);
			$rulentry =~ s/^\s*//;
			$pattern = $rulentry;
			if ($last_selector ne $selector) {	# Update last selector
				$last_selector = $selector;
			}
			$selector =~ s/:$//;			# Remove final ':' on selector
			$* = 1;							# There can be multi-line matching
			do add_log ("selector '$selector', pattern '$pattern'")
				if $loglvl > 19;
			unless ($Matched{$selector}) {	# Identical selectors are OR'ed
				$Matched{$selector} = do match($selector, $pattern);
			}
		}
		$matched = 1;						# Assume everything matched
		foreach $key (keys %Matched) {
			$matched = 0 unless $Matched{$key};
		}
		if ($matched) {						# Execute action if pattern matched
			# Make sure the rule has not already been executed in that mode
			if ($Executed[$i] =~ /,$wmode,/) {
				do add_log("NOTICE loop detected, rule $j, state $wmode")
					if $loglvl > 5;
				last rule;					# Processing ends here
			} else {						# Rule was never executed
				$Executed[$i] = ',' unless $Executed[$i];
				$Executed[$i] .= "$wmode,";
			}
			$ever_matched = 1;				# At least one match
			&add_log("MATCH on rule #$j in mode $wmode") if $loglvl > 8;
			&track_rule($j, $wmode) if $track_all;
			&s_match($j, $wmode);			# Record match for statistics
			$status = do xeqte($action);
			last rule if $status == $FT_CONT;
			$ever_matched = 0;				# No match if REJECT or RESTART
			next rule if $status == $FT_REJECT;
			$i = -1;		# Restart analysis from the beginning ($FT_RESTART)
		}
	}

	# Deal with vacation mode. It applies only on mail not previously seen.
	# The vacation mode must be turned on in the configuration file. The
	# conditions for a vacation message to be sent are:
	#   - Message was directly sent to the user.
	#   - Message does not come from a special user like root.
	#   - Vacation message was not disabled via a VACATION command

	if (!$ever_seen && $cf'vacation =~ /on/i && $vacation) {
		unless (&special_user) {	# Not from special user and sent to me
			# Send vacation message only once per address per period
			&xeqte("ONCE (%r,vacation,$cf'vacperiod) MESSAGE $cf'vacfile");
			&s_vacation;		# Message received while in vacation
		}
	}

	# Default action if no rule ever matched
	unless ($ever_matched) {
		do add_log("NOTICE no match, leaving in mailbox") if $loglvl > 5;
		do xeqte("LEAVE");			# Default action anyway
		&s_default;					# One more application of default rule
	} else {
		unless ($ever_saved) {
			do add_log("NOTICE not saved, leaving in mailbox") if $loglvl > 5;
			do xeqte("LEAVE");		# Leave if message not saved
			&s_saved;				# Message saved by default rule
		}
	}
	&s_filtered($Header{'Length'});		# Update statistics
	0;									# Ok status
}

# Parse the mail and fill-in the Header associative array. The special entries
# All, Body and Head respectively hold the whole message, the body and the
# header of the message.
sub parse_mail {
	local($file_name) = shift(@_);	# Where mail is stored ("" for stdin)
	local($last_header) = "";		# Name of last header (for continuations)
	local($first_from) = "";		# The first From line in mails
	local($lines) = 0;				# Number of lines in the body
	local($length) = 0;				# Length of body, in bytes
	local($last_was_nl) = 0;		# True when last line was a '\n'
	local($fd) = STDIN;				# Where does the mail come from ?
	local($_);
	undef %Header;					# Reset the all structure holding message

	if ($file_name ne '') {			# Mail spooled in a file
		unless(open(MAIL, $file_name)) {
			&add_log("ERROR cannot open $file_name: $!");
			return;
		}
		$fd = MAIL;
	}
	$Userpath = "";					# Reset path from possible previous @PATH 

	# Pre-extend 'All', 'Body' and 'Head'
	$Header{'All'} = ' ' x 5000;
	$Header{'Body'} = ' ' x 4500;
	$Header{'Head'} = ' ' x 500;
	$Header{'All'} = '';
	$Header{'Body'} = '';
	$Header{'Head'} = '';

	do add_log ("parsing mail") if $loglvl > 18;
	while (<$fd>) {
		$Header{'All'} .= $_;
		if (1../^$/) {						# EOH is a blank line
			next if /^$/;					# Skip EOH marker
			$Header{'Head'} .= $_;			# Record line in header

			if (/^\s/) {					# It is a continuation line
				s/^\s+/ /;					# Swallow multiple spaces
				chop;						# Remove final new-line
				$Header{$last_header} .= "\n$_" if $last_header ne '';
				do add_log("WARNING bad continuation in header, line $.")
					if $last_header eq '' && $loglvl > 4;
			} elsif (/^([\w-]+):\s*(.*)/) {	# We found a new header
				# Guarantee only one From: header line. If multiple From: are
				# found, keep the last one.
				# Multiple headers like 'Received' are separated by a new-
				# line character. All headers end on a non new-line.
				$last_header = $1;
				if ($1 eq 'From' && defined($Header{$1})) {
					$Header{$1} = $2;
					do add_log("WARNING duplicate From in header, line $.")
						if $loglvl > 4;
				} elsif ($Header{$1} ne '') {
					$Header{$1} .= "\n$2";
				} else {
					$Header{$1} .= $2;
				}
			} elsif (/^From\s+(\S+)/) {		# The very first From line
				$first_from = $1;
			}

		} else {
			$lines++;								# One more line in body
			$length += length($_);					# Update length of message
			s/^From(\s)/>From$1/ if $last_was_nl;	# Escape From keyword
			$last_was_nl = /^$/;					# Keep track of single '\n'
			$Header{'Body'} .= $_;
			chop;
			# Deal with builtin commands
			if (s/^@(\w+)\s*//) {					# A builtin command ?
				local($subroutine) = $Builtin{$1};
				eval("do $subroutine('$_')");
			}
		}
	}
	close MAIL if $file_name ne '';
	do header_check($first_from, $lines);	# Sanity checks
}

# Now do some sanity checks:
# - if there is no From: header, fill it in with the first From
# - if there is no To: but an Apparently-To:, copy it also as a To:
#
# We guarantee the following header entries:
#   From:         the value of the From field
#   To:           to whom the mail was sent
#   Lines:        number of lines in the message
#   Length:       number of bytes in the message
#   Reply-To:     the address we may use to reply
#   Sender:       the actual sender, even if same as From

sub header_check {
	local($first_from, $lines) = @_;	# First From line, number of lines
	unless (defined $Header{'From'}) {
		$Header{'From'} = $first_from;
	}

	# There is usually one Apparently-To line per address. Remove all new lines
	# in the header line and replace them with ','.
	$* = 1;
	$Header{'Apparently-To'} =~ s/\n/,/g;	# Remove new-lines
	$* = 0;
	$Header{'Apparently-To'} =~ s/,$/\n/;	# Restore last new-line

	# If no To: field, then maybe there is an Apparently-To: instead. If so,
	# make them identical. Otherwise, assume the mail was directed to the user.
	if (!$Header{'To'} && $Header{'Apparently-To'}) {
		$Header{'To'} = $Header{'Apparently-To'};
	}
	unless ($Header{'To'}) {
		do add_log("WARNING no To: field, assuming $cf'user") if $loglvl > 4;
		$Header{'To'} = $cf'user;
	}

	# Set number of lines in body, unless there is already a Lines:
	# header in which case we trust it. Same for Length.
	$Header{'Lines'} = $lines unless defined($Header{'Lines'});
	$Header{'Length'} = $length unless defined($Header{'Length'});

	# If there is no Reply-To: line, then take the return path, if any.
	# Otherwise use the address found in From.
	if (!$Header{'Reply-To'}) {
		local($tmp) = $Header{'Return-Path'};
		$tmp =~ /<(.*)>/ && ($tmp = $1);		# Remove the <> in path
		$Header{'Reply-To'} = $tmp if $tmp;
		$Header{'Reply-To'} = (&parse_address($Header{'From'}))[0] unless $tmp;
	}

	# Unless there is already a sender line, fake one using From field
	if (!$Header{'Sender'}) {
		$Header{'Sender'} = $first_from;
	}
}

# Return true if the mail was from a special user (root, uucp...) or if the
# mail was not directly mailed to the user (i.e. it comes from a distribution
# list or has bounced somewhere).
sub special_user {
	# Before sending the vacation message, we have to make sure the mail
	# was sent to the user directly, through a 'To:' or a 'Cc:'. Otherwise,
	# it must be from a mailing list or a 'Bcc:' and we don't want to
	# send something back in that case.
	local($matched) = do match_list("To", $cf'user);
	$matched = do match_list("Cc", $cf'user) unless $matched;
	unless ($matched) {
		do add_log("mail was not directly sent to $cf'user") if $loglvl > 8;
		return 1;
	}
	# If there is a Precedence: header set to either 'bulk' or 'junk', then
	# we do not reply either.
	local($prec) = $Header{'Precedence'};
	if ($prec =~ /^bulk|junk/i) {
		do add_log("mail was tagged with a '$prec' precedence") if $loglvl > 8;
		return 1;
	}
	# Make sure the mail does not come from a "special" user, as listed in
	# the %Special array (root, uucp...)
	$matched = 0;
	local($matched_login);
	foreach $login (keys %Special) {
		$matched = do match_single("From", $login);
		$matched_login = $login if $matched;
		last if $matched;
	}
	if ($matched) {
		do add_log("mail was from special user $matched_login")
			if $loglvl > 8;
		return 1;
	}
}

# Log reception of mail (sender and subject fields). This is mainly intended
# for people like me who parse the logfile once in a while to do more 
# statistics about mail reception. Hence the another distinction between
# original mails and answers.
sub reception {
	local($subject) = $Header{'Subject'};
	local($sender) = $Header{'Sender'};
	local($from) = $Header{'From'};
	&add_log("FROM $from");
	&add_log("VIA $sender")
		if $sender ne '' && $sender ne (&parse_address($from))[0];
	if ($subject ne '') {
		if ($subject =~ s/^Re:\s*//) {
			&add_log("REPLY $subject");
		} else {
			&add_log("ABOUT $subject");
		}
	}
	print "-------- From $from\n" if $track_all;
}

# Print match on STDOUT when -t option is used
sub track_rule {
	local($number, $mode) = @_;
	print "*** Match on rule $number in mode $mode ***\n";
	&print_rule($number);
}

