;# $Id: filter.pl,v 2.9 92/07/14 16:49:57 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	filter.pl,v $
;# Revision 2.9  92/07/14  16:49:57  ram
;# 3.0 beta baseline.
;# 
;#
#
# Filter commands are run from here
#

# The following variables are inherited from run_command:
#  $mfile is the name of the mail file processed
#  $cmd is the command to be run
#  $cmd_name is the command name (upper-cased)
#  $ever_saved which states whether a saving/discarding action occurred
#  $cont is the continuation status, modified by REJECT and friends
#  $vacation which is a boolean stating whether vacation messages are allowed
# The working mode is held in $wmode (comes from analyze_mail).
# All the commands return an exit status: 0 for ok, 1 for failure.

# Run the PROCESS command
sub run_process {
	if (0 != do process()) {
		do add_log("ERROR while processing [$mfile]--queing it")
			if ($loglvl > 0);
		do queue_mail($file_name);
		return 1;
	}
	do add_log("PROCESSED [$mfile]") if $loglvl > 8;
	0;
}

# Run the LEAVE command
sub run_leave {
	local($mbox, $failed) = do leave();
	unless ($failed) {
		do add_log("LEFT [$mfile] in mailbox") if $loglvl > 2;
	}
	# Even if it failed, mark it as saved anyway, as the default action would
	# be a saving in mailbox and there is little chance another attempt would
	# succeed while this one failed.
	$ever_saved = 1;		# At least we tried to save it
	$failed;
}

# Run the SAVE command
sub run_save {
	local($folder) = $cmd =~ /^\w+\s+(\S+)/;	# Get first parameter
	&save_message($folder);
}

# Run the STORE command
sub run_store {
	local($folder) = $cmd =~ /^\w+\s+(\S+)/;	# Get first parameter
	local($mbox, $failed, $log_message) = &run_saving($folder, $FOLDER_APPEND);
	unless ($failed) {
		$ever_saved = 1;			# We were able to save it
		($mbox, $failed) = do leave();
		unless ($failed) {
			do add_log("STORED [$mfile] in $log_message") if $loglvl > 2;
		} else {
			do add_log("WARNING only SAVED [$mfile] in $log_message")
				if $loglvl > 1;
			return 1;
		}
	} else {
		($mbox, $failed) = do leave();
		unless ($failed) {
			$ever_saved = 1;			# We were able to save it
			do add_log("WARNING only LEFT [$mfile] in mailbox")
				if $loglvl > 1;
		}
	}
	$failed;
}

# Run the WRITE command
sub run_write {
	local($folder) = $cmd =~ /^\w+\s+(\S+)/;	# Get first parameter
	local($mbox, $failed, $log_message) = &run_saving($folder, $FOLDER_REMOVE);
	unless ($failed) {
		do add_log("WROTE [$mfile] in $log_message") if $loglvl > 2;
		$ever_saved = 1;			# We were able to save it
	}
	$failed;
}

# Run the DELETE command
sub run_delete {
	do add_log("DELETED [$mfile]") if $loglvl > 2;
	$ever_saved = 1;		# User chose to discard it, it counts as a save
	0;
}

# Run the MESSAGE command
sub run_message {
	local($msg) = $cmd =~ m|^\w+\s+(\S+)|;	# Vacation message location
	$msg =~ s/~/$cf'home/g;					# ~ substitution
	local($failed) = do message($msg);
	unless ($failed) {
		$msg =~ s|^$cf'home|~|;				# Replace the home directory by ~
		do add_log("MESSAGE $msg for [$mfile]") if $loglvl > 2;
	}
	$failed;
}

# Run the NOTIFY command
sub run_notify {
	local($address, $msg) = $cmd =~ m|^\w+\s+(\S+)\s+(\S+)|;
	$msg =~ s/~/$cf'home/g;					# ~ substitution
	local($failed) = do notify($msg, $address);
	unless ($failed) {
		$msg =~ s|^$cf'home|~|;				# Replace the home directory by ~
		do add_log("NOTIFIED $msg for [$mfile]") if $loglvl > 2;
	}
	$failed;
}

# Run the REJECT command
sub run_reject {
	$cont = $FT_REJECT;			# Reject ($cont defined in run_command)
	do add_log("REJECTED [$mfile] in state $wmode") if $loglvl > 4;
	0;
}

# Run the RESTART command
sub run_restart {
	$cont = $FT_RESTART;		# Restart ($cont defined in run_command)
	do add_log("RESTARTED [$mfile] in state $wmode") if $loglvl > 4;
	0;
}

# Run the ABORT command
sub run_abort {
	$cont = $FT_ABORT;			# Abort filtering ($cont defined in run_command)
	do add_log("ABORTED [$mfile] in state $wmode") if $loglvl > 4;
	0;
}

# Run the RESYNC command
sub run_resync {
	do header_resync();			# Resynchronize the %Header array
	do add_log("RESYNCED [$mfile]") if $loglvl > 4;
	0;
}

# Run the BEGIN command
sub run_begin {
	local($newstate) = $cmd =~ m|^\w+\s+(\S+)|;	# New state wanted
	$newstate = 'INITIAL' unless $newstate;
	$wmode = $newstate;			# $wmode comes from analyze_mail
	do add_log("BEGUN new state $newstate") if $loglvl > 4;
	0;
}

# Run the RECORD command
sub run_record {
	local($option) = $cmd =~ m|^\w+\s+-(\w)|;
	if (&history_record) {		# Message already seen
		$wmode = '_SEEN_';		# Enter special mode ($wmode from analyze_mail)
		&add_log("NOTICE entering seen mode") if $loglvl > 5;
		&alter_execution($option);
	}
	&add_log("RECORDED [$mfile]") if $loglvl > 4;
	0;
}

# Run the UNIQUE command
sub run_unique {
	local($option) = $cmd =~ m|^\w+\s+-(\w)|;
	if (&history_record) {		# Message already seen
		&add_log("NOTICE message tagged as saved") if $loglvl > 5;
		$ever_saved = 1;		# In effect, runs a DELETE
		&alter_execution($option);
	}
	&add_log("UNIQUE [$mfile]") if $loglvl > 4;
	0;
}

# Run the FORWARD command
sub run_forward {
	local($addresses) = $cmd =~ m|^\w+\s+(.*)|;	# Address(es)
	local($failed) = do forward($addresses);
	unless ($failed) {
		do add_log("FORWARDED [$mfile] to $addresses") if $loglvl > 2;
		$ever_saved = 1;		# Forwarding succeeded, counts as a save
	}
	$failed;
}

# Run the BOUNCE command
sub run_bounce {
	local($addresses) = $cmd =~ m|^\w+\s+(.*)|;	# Address(es)
	local($failed) = do bounce($addresses);
	unless ($failed) {
		do add_log("BOUNCED [$mfile] to $addresses") if $loglvl > 2;
		$ever_saved = 1;		# Bouncing succeeded, counts as a save
	}
	$failed;
}

# Run the POST command
sub run_post {
	local($newsgroups) = $cmd =~ m|^\w+\s+(.*)|;	# Newsgroup(s)
	local($failed) = do post($newsgroups);
	unless ($failed) {
		do add_log("POSTED [$mfile] to $newsgroups") if $loglvl > 2;
		$ever_saved = 1;		# Posting succeeded, counts as a save
	}
	$failed;
}

# Run the RUN command
sub run_run {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $NO_INPUT, $NO_FEEDBACK);
	unless ($failed) {
		do add_log("RAN '$program' for [$mfile]") if $loglvl > 4;
	}
	$failed;
}

# Run the PIPE command
sub run_pipe {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $MAIL_INPUT, $NO_FEEDBACK);
	unless ($failed) {
		do add_log("PIPED [$mfile] to '$program'") if $loglvl > 4;
	}
	$failed;
}

# Run the GIVE command
sub run_give {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $BODY_INPUT, $NO_FEEDBACK);
	unless ($failed) {
		do add_log("GAVE [$mfile] to '$program'") if $loglvl > 4;
	}
	$failed;
}

# Run the PASS command
sub run_pass {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $BODY_INPUT, $FEEDBACK);
	unless ($failed) {
		do add_log("PASSED [$mfile] through '$program'") if $loglvl > 4;
	}
	$failed;
}

# Run the FEED command
sub run_feed {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $MAIL_INPUT, $FEEDBACK);
	unless ($failed) {
		do add_log("FED [$mfile] through '$program'") if $loglvl > 4;
	}
	$failed;
}

# Run the PURIFY command
sub run_purify {
	local($program) = $cmd =~ m|^\w+\s+(.*)|;	# Program to run
	local($failed) = do shell_command($program, $HEADER_INPUT, $FEEDBACK);
	unless ($failed) {
		do add_log("PURIFIED [$mfile] through '$program'") if $loglvl > 4;
	}
	$failed;
}

# Run the BACK command
# Manipulates dynamically bound variable $cont (output from xeqte)
sub run_back {
	# BACK command is handled recursively. The local variable $Back will be set
	# by xeq_back() if any feedback is to ever occur. This routine will be
	# transparently called instead of the usual handle_output() because of the
	# dynamic aliasing done here.
	local($Back) = '';					# BACK may be nested
	local(*handle_output) = *xeq_back;	# Any output to be put in $Back
	local($command) = $cmd =~ m|^BACK\s+(.*)|;
	$command =~ s/%/%%/g;				# Protect against 2nd macro substitution
	&run_command($command);				# Run command (ignore return value)
	if ($Back ne '') {
		&add_log("got '$Back' back") if $loglvl > 11;
		$cont = &xeqte($Back);			# Get continuation status back
		$@ = '';						# Avoid cascade of (same) error report
		unless ($failed) {
			&add_log("BACK from '$command'") if $loglvl > 4;
		}
	} else {
		&add_log("WARNING got nothing out of '$command'") if $loglvl > 5;
	}
	0;			# Never fails
}

# Run the ONCE command
sub run_once {
	local($_) = $cmd;					# The whole command line
	local($hname);						# Hash name (e-mail address)
	local($tag);						# Tag associated with command
	local($raw_period);					# The period, as written
	if (s/^ONCE\s+\(([^,\)]*),\s*([^,;\)]*),\s*(\w+)\s*\)//) {
		($hname, $tag, $raw_period) = ($1, $2, $3);
		do add_log("tag is ($hname, $tag, $raw_period)") if $loglvl > 18;
	} else {
		do add_log("ERROR bad once syntax (invalid tag)") if $loglvl > 1;
		return 1;
	}
	s/^\s*//;							# Remove leading spaces
	local($period) = do seconds_in_period($raw_period);
	do add_log("period is $raw_period = $period seconds") if $loglvl > 18;
	if (&once_check($hname, $tag, $period)) {
		do add_log("ONCE ($hname, $tag, $raw_period) $_") if $loglvl > 7;
		&s_once($cmd_name, $wmode, $tag);
		s/%/%%/g;						# Protect against 2nd macro substitution
		do run_command($_);				# Run command and propagate status
	} else {
		do add_log("retry time not reached for $_") if $loglvl > 12;
		&s_noretry($cmd_name, $wmode, $tag);
		0;								# Ok status
	}
}

# Run the SELECT command
sub run_select {
	local($_) = $cmd;					# The whole command line
	local($start, $end);				# Date strings for start and end
	if (s/^SELECT\s+\(([^.\)]*)\.\.\s*([^\)]*)\)//) {
		($start, $end) = ($1, $2);
		$start =~ s/\s*$//;				# Remove trailing spaces
		$end =~ s/\s*$//;
		&add_log("time is ($start .. $end)") if $loglvl > 18;
	} else {
		&'add_log("ERROR bad select syntax (invalid time)") if $loglvl > 1;
		return 1;
	}
	local($now) = time;					# Current time
	local($sec_start, $sec_end);		# Start and end converted in seconds
	$sec_start = &getdate($start, $now);
	if ($sec_start == -1) {
		&add_log("ERROR in SELECT: 1st time '$start'") if $loglvl > 1;
		return 1;
	}
	$sec_end = &getdate($end, $now);
	if ($sec_end == -1) {
		&add_log("ERROR in SELECT: 2nd time '$end'") if $loglvl > 1;
		return 1;
	}
	if ($sec_start > $sec_end) {
		&add_log("WARNING time selection always impossible?") if $loglvl > 1;
		return 0;
	}
	s/^\s*//;							# Remove leading spaces
	if ($now >= $sec_start && $now <= $sec_end) {
		&add_log("SELECT ($start .. $end) $_") if $loglvl > 7;
		s/%/%%/g;						# Protect against 2nd macro substitution
		&run_command($_);				# Run command and propagate status
	} else {
		&add_log("time period not good for $_") if $loglvl > 12;
		0;								# Ok status
	}
}

# Run the NOP command
sub run_nop {
	do add_log("NOP [$mfile]") if $loglvl > 7;
	0;
}

# Run the STRIP command
sub run_strip {
	local($headers) = $cmd =~ m|^\w+\s+(.*)|;	# Headers to remove
	do alter_header($headers, $HD_STRIP);
	$headers = join(', ', split(/\s/, $headers));
	do add_log("STRIPPED $headers from [$mfile]") if $loglvl > 7;
	0;
}

# Run the KEEP command
sub run_keep {
	local($headers) = $cmd =~ m|^\w+\s+(.*)|;	# Headers to keep
	do alter_header($headers, $HD_KEEP);
	$headers = join(', ', split(/\s/, $headers));
	do add_log("KEPT $headers from [$mfile]") if $loglvl > 7;
	0;
}

# Run the ANNOTATE command
sub run_annotate {
	local($date, $field, $value) = $cmd =~ m|^\w+\s+(-d\s+)?([\w\-]+):?\s*(.*)|;
	if (0 == &annotate_header($field, $value, $date)) {
		&add_log("ANNOTATED [$mfile] with $field") if $loglvl > 7;
	}
	0;
}

# Run the ASSIGN command
sub run_assign {
	local($var, $value) = $cmd =~ m|^\w+\s+(\w+)\s+(.*)|;
	$Variable{$var} = $value;
	do add_log("ASSGINED '$value' to '$var' [$mfile]") if $loglvl > 7;
	0;
}

# Run the TR command
sub run_tr {
	local($variable, $tr) = $cmd =~ m|^\w+\s+([#\w]+)\s+(.*)|;
	&alter_value($variable, "tr$tr");
}

# Run the SUBST command
sub run_subst {
	local($variable, $s) = $cmd =~ m|^\w+\s+([#\w]+)\s+(.*)|;
	&alter_value($variable, "s$s");
}

# Run the SPLIT command
sub run_split {
	local($folder) = $cmd =~ m|^\w+\s+(.*)|;	# Folder where split occurs
	local($failed) = do split($folder);
	if (0 == $failed % 2) {			# Message was in digest format
		if ($failed & 0x4) {
			do add_log("SPLIT [$mfile] in mailagent's queue") if $loglvl > 2;
		} else {
			do add_log("SPLIT [$mfile] in $folder") if $loglvl > 2;
		}
		# If digest was not in RFC-934 style, there is a chance the split
		# was not correctly performed. To avoid any accidental loss of
		# information, the original digest message is also saved if SPLIT
		# had a folder argument, or it is not tagged saved.
		if ($failed & 0x8) {		# Digest was not RFC-934 compliant
			&add_log("NOTICE [$mfile] not RFC-934 compliant") if $loglvl > 6;
			if ($folder ne '') {
				&add_log("NOTICE saving original [$mfile] in $folder")
					if $loglvl > 6;
				&save_message($folder);
			} else {
				&add_log("NOTICE [$mfile] not tagged as saved")
					if $loglvl > 6 && ($failed & 0x2);
			}
		} else {
			$ever_saved = 1 if $failed & 0x2;	# Split -i succeeded
		}
		$failed = 0;
	}
	# If message was not in digest format and a folder was specified, save
	# message in that folder.
	if ($failed < 0 && $folder ne '') {
		&add_log("NOTICE [$mfile] not in digest format") if $loglvl > 6;
		$failed = &save_message($folder);
	}
	$failed ? 1 : 0;	# Failure status from split can be negative
}

# Run the VACATION command
sub run_vacation {
	return 0 unless $cf'vacation =~ /on/i;	# Ignore if vacation mode off
	local($mode) = $cmd =~ m|^\w+\s+(.*)|;	# Vacation mode
	$vacation = ($mode =~ /on/i) ? 1 : 0;
	$mode = $vacation ? 'on' : 'off';
	&add_log("vacation message turned $mode") if $loglvl > 7;
	0;
}

# Run the QUEUE command
sub run_queue {
	# Mail is saved as a 'qm' file, to avoid endless loops when mailagent
	# processes the queue. This means the mail will be deferred for at
	# least half an hour.
	local($failed) = &queue_mail('', 1);	# No file name, mail in %Header
	$ever_saved = 1 unless $failed;			# Queuing counts as saving
}

# For SAVE, STORE or WRITE, the job is the same
sub run_saving {
	# If the name is not an absolute path, the folder directory is taken
	# in the "maildir" environment variable. If none, defaults to ~/Mail.
	local($folder, $remove) = @_;				# Shall we remove folder first?
	local($folddir) = $XENV{'maildir'};			# Folder directory location
	$folder = "~/mbox" unless $folder;			# No folder -> save in mbox
	$folder =~ s/~/$cf'home/g;					# ~ substitution
	$folddir =~ s/~/$cf'home/g;					# ~ substitution
	$folddir = "$cf'home/Mail" unless $folddir;	# Default folders in ~/Mail
	$folder = "$folddir/$folder" unless $folder =~ m|^/|;
	local($dir) = $folder =~ m|(.*)/.*|;		# Get directory name
	unless (-d "$dir") {
		do makedir($dir);
		unless (-d "$dir") {
			do add_log("ERROR couldn't create directory $dir")
				if $loglvl > 0;
		} else {
			do add_log("created directory $dir") if $loglvl > 7;
		}
	}
	if ($remove == $FOLDER_REMOVE) {
		# Folder has to be removed before writting into it. However, if it
		# is write protected, do not unlink it (save will fail later on anyway).
		unlink "$folder" if -f "$folder" && -w _;
	}
	local($mbox, $failed) = do save($folder);
	local($log_message);				# Log message to be issued
	unless ($failed) {
		local($dir, $file) = $folder =~ m|(.*)/(.*)|;
		if ($dir eq $folddir) {
			$log_message = "folder $file";
		} else {
			$folder =~ s|^$cf'home|~|;	# Replace the home directory by ~
			$log_message = "$folder";
		}
	}

	# Return the status of the save command and a part of the logging message
	# to be issued. That way, we get a nice contextual log.
	($mbox, $failed, $log_message);
}

# Perform the appropriate continuation status, depending on the option:
sub alter_execution {
	local($option) = @_;		# The invocation option
	if ($option eq 'r') {		# Asks for RESTART
		&run_restart;
	} elsif ($option eq 'a') {	# Asks for ABORT
		&run_abort;
	} else {					# Default is to REJECT
		&run_reject;
	}
}

# Save message in specified folder
sub save_message {
	local($folder) = @_;
	local($mbox, $failed, $log_message) = &run_saving($folder, $FOLDER_APPEND);
	unless ($failed) {
		do add_log("SAVED [$mfile] in $log_message") if $loglvl > 2;
		$ever_saved = 1;			# We were able to save it
	}
	$failed;
}

