;# $Id: history.pl,v 2.9 92/07/14 16:50:08 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	history.pl,v $
;# Revision 2.9  92/07/14  16:50:08  ram
;# 3.0 beta baseline.
;# 
;# 
;# Handle the message history mechanism, which is used to reject duplicates.
;# Each message-id tag is stored in a file, along with a time-stamp (to enable
;# its removal after a given period.
;#
# Record message whose message ID is given as argument and return 0 if the
# message was recorded for the first time or if there is no valid message ID.
# Return 1 if the message was already recorded, and hence was already seen.
sub history_record {
	local($msg_id) = $Header{'Message-Id'};		# Message-ID header
	# If there is no message ID, use the concatenation of date + from fields.
	if ($msg_id) {
		# Keep only the ID stored within <> brackets
		($msg_id) = $msg_id =~ m|^<(.*)>\s*$|;
	} else {
		# Use date + from iff there is a date. We cannot use the from field
		# alone, obviously!!
		$msg_id = $Header{'Date'};
		$msg_id .= $Header{'From'} if $msg_id;
	}
	$msg_id =~ s/\s//g;				# Suppress all spaces
	return 0 unless $msg_id;		# Cannot record message without an ID
	# Hashing of the message ID is done based on the two first letters of
	# the host name (assuming message ID has the form whatever@host).
	local($stamp, $host) = $msg_id =~ m|^(.*)@([.\w]+)|;
	local($time, $line) = &dbr'info($host, 'HISTORY', $stamp);
	return 0 if $time == -1;				# An error occurred
	if ($time > 0) {						# Message already recorded
		&add_log("history duplicate <$msg_id>") if $loglvl > 6;
		return 1;
	}
	&dbr'update($host, 'HISTORY', 0, $stamp);	# Record message (appending)
	0;			# First time ever seen
}

